#!/usr/bin/python2.4
# Generated by the protocol buffer compiler.  DO NOT EDIT!

from google.protobuf import descriptor
from google.protobuf import message
from google.protobuf import reflection
from google.protobuf import service
from google.protobuf import service_reflection
from google.protobuf import descriptor_pb2
_ERRORCODE = descriptor.EnumDescriptor(
  name='ErrorCode',
  full_name='sae.ErrorCode',
  filename='ErrorCode',
  values=[
    descriptor.EnumValueDescriptor(
      name='ERR_OK', index=0, number=0,
      options=None,
      type=None),
    descriptor.EnumValueDescriptor(
      name='ERR_INTERNAL', index=1, number=1,
      options=None,
      type=None),
    descriptor.EnumValueDescriptor(
      name='ERR_INVALID_METHOD', index=2, number=2,
      options=None,
      type=None),
    descriptor.EnumValueDescriptor(
      name='ERR_INVALID_TRANSACTION', index=3, number=3,
      options=None,
      type=None),
    descriptor.EnumValueDescriptor(
      name='ERR_TRANSACTION_EXISTS', index=4, number=4,
      options=None,
      type=None),
    descriptor.EnumValueDescriptor(
      name='ERR_UNKNOWN_ACTIVATOR', index=5, number=5,
      options=None,
      type=None),
    descriptor.EnumValueDescriptor(
      name='ERR_INVALID_PROFILE', index=6, number=6,
      options=None,
      type=None),
    descriptor.EnumValueDescriptor(
      name='ERR_INVALID_SCHEME', index=7, number=7,
      options=None,
      type=None),
    descriptor.EnumValueDescriptor(
      name='ERR_UNKNOWN_EVENT_SOURCE', index=8, number=8,
      options=None,
      type=None),
    descriptor.EnumValueDescriptor(
      name='ERR_AUTH_FAILED', index=9, number=9,
      options=None,
      type=None),
    descriptor.EnumValueDescriptor(
      name='ERR_AUTH_REQUIRED', index=10, number=10,
      options=None,
      type=None),
    descriptor.EnumValueDescriptor(
      name='ERR_INVALID_UPGRADE', index=11, number=11,
      options=None,
      type=None),
    descriptor.EnumValueDescriptor(
      name='ERR_OVERLOAD', index=12, number=12,
      options=None,
      type=None),
    descriptor.EnumValueDescriptor(
      name='ERR_INVALID_SCRIPT', index=13, number=13,
      options=None,
      type=None),
    descriptor.EnumValueDescriptor(
      name='ERR_SCRIPT_EXCEPTION', index=14, number=14,
      options=None,
      type=None),
    descriptor.EnumValueDescriptor(
      name='ERR_ACTIVATOR_NOT_AVAILABLE', index=15, number=15,
      options=None,
      type=None),
    descriptor.EnumValueDescriptor(
      name='ERR_DOWN', index=16, number=16,
      options=None,
      type=None),
  ],
  options=None,
)


_ACCESSSCHEME = descriptor.EnumDescriptor(
  name='AccessScheme',
  full_name='sae.AccessScheme',
  filename='AccessScheme',
  values=[
    descriptor.EnumValueDescriptor(
      name='TELNET', index=0, number=0,
      options=None,
      type=None),
    descriptor.EnumValueDescriptor(
      name='SSH', index=1, number=1,
      options=None,
      type=None),
    descriptor.EnumValueDescriptor(
      name='HTTP', index=2, number=2,
      options=None,
      type=None),
  ],
  options=None,
)


ERR_OK = 0
ERR_INTERNAL = 1
ERR_INVALID_METHOD = 2
ERR_INVALID_TRANSACTION = 3
ERR_TRANSACTION_EXISTS = 4
ERR_UNKNOWN_ACTIVATOR = 5
ERR_INVALID_PROFILE = 6
ERR_INVALID_SCHEME = 7
ERR_UNKNOWN_EVENT_SOURCE = 8
ERR_AUTH_FAILED = 9
ERR_AUTH_REQUIRED = 10
ERR_INVALID_UPGRADE = 11
ERR_OVERLOAD = 12
ERR_INVALID_SCRIPT = 13
ERR_SCRIPT_EXCEPTION = 14
ERR_ACTIVATOR_NOT_AVAILABLE = 15
ERR_DOWN = 16
TELNET = 0
SSH = 1
HTTP = 2



_MESSAGE = descriptor.Descriptor(
  name='Message',
  full_name='sae.Message',
  filename='sae.proto',
  containing_type=None,
  fields=[
    descriptor.FieldDescriptor(
      name='id', full_name='sae.Message.id', index=0,
      number=1, type=13, cpp_type=3, label=2,
      default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='request', full_name='sae.Message.request', index=1,
      number=2, type=11, cpp_type=10, label=1,
      default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='response', full_name='sae.Message.response', index=2,
      number=3, type=11, cpp_type=10, label=1,
      default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='error', full_name='sae.Message.error', index=3,
      number=4, type=11, cpp_type=10, label=1,
      default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],  # TODO(robinson): Implement.
  enum_types=[
  ],
  options=None)


_REQUEST = descriptor.Descriptor(
  name='Request',
  full_name='sae.Request',
  filename='sae.proto',
  containing_type=None,
  fields=[
    descriptor.FieldDescriptor(
      name='method', full_name='sae.Request.method', index=0,
      number=1, type=9, cpp_type=9, label=2,
      default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='serialized_request', full_name='sae.Request.serialized_request', index=1,
      number=2, type=12, cpp_type=9, label=2,
      default_value="",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],  # TODO(robinson): Implement.
  enum_types=[
  ],
  options=None)


_RESPONSE = descriptor.Descriptor(
  name='Response',
  full_name='sae.Response',
  filename='sae.proto',
  containing_type=None,
  fields=[
    descriptor.FieldDescriptor(
      name='serialized_response', full_name='sae.Response.serialized_response', index=0,
      number=1, type=12, cpp_type=9, label=1,
      default_value="",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],  # TODO(robinson): Implement.
  enum_types=[
  ],
  options=None)


_ERROR = descriptor.Descriptor(
  name='Error',
  full_name='sae.Error',
  filename='sae.proto',
  containing_type=None,
  fields=[
    descriptor.FieldDescriptor(
      name='code', full_name='sae.Error.code', index=0,
      number=1, type=14, cpp_type=8, label=2,
      default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='text', full_name='sae.Error.text', index=1,
      number=2, type=9, cpp_type=9, label=1,
      default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],  # TODO(robinson): Implement.
  enum_types=[
  ],
  options=None)


_ACCESSPROFILE = descriptor.Descriptor(
  name='AccessProfile',
  full_name='sae.AccessProfile',
  filename='sae.proto',
  containing_type=None,
  fields=[
    descriptor.FieldDescriptor(
      name='profile', full_name='sae.AccessProfile.profile', index=0,
      number=1, type=9, cpp_type=9, label=2,
      default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='scheme', full_name='sae.AccessProfile.scheme', index=1,
      number=2, type=14, cpp_type=8, label=2,
      default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='address', full_name='sae.AccessProfile.address', index=2,
      number=3, type=9, cpp_type=9, label=2,
      default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='port', full_name='sae.AccessProfile.port', index=3,
      number=4, type=13, cpp_type=3, label=1,
      default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='user', full_name='sae.AccessProfile.user', index=4,
      number=5, type=9, cpp_type=9, label=1,
      default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='password', full_name='sae.AccessProfile.password', index=5,
      number=6, type=9, cpp_type=9, label=1,
      default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='super_password', full_name='sae.AccessProfile.super_password', index=6,
      number=7, type=9, cpp_type=9, label=1,
      default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='path', full_name='sae.AccessProfile.path', index=7,
      number=8, type=9, cpp_type=9, label=1,
      default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='snmp_ro', full_name='sae.AccessProfile.snmp_ro', index=8,
      number=9, type=9, cpp_type=9, label=1,
      default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='snmp_rw', full_name='sae.AccessProfile.snmp_rw', index=9,
      number=10, type=9, cpp_type=9, label=1,
      default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],  # TODO(robinson): Implement.
  enum_types=[
  ],
  options=None)


_PINGREQUEST = descriptor.Descriptor(
  name='PingRequest',
  full_name='sae.PingRequest',
  filename='sae.proto',
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],  # TODO(robinson): Implement.
  enum_types=[
  ],
  options=None)


_PINGRESPONSE = descriptor.Descriptor(
  name='PingResponse',
  full_name='sae.PingResponse',
  filename='sae.proto',
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],  # TODO(robinson): Implement.
  enum_types=[
  ],
  options=None)


_REGISTERREQUEST = descriptor.Descriptor(
  name='RegisterRequest',
  full_name='sae.RegisterRequest',
  filename='sae.proto',
  containing_type=None,
  fields=[
    descriptor.FieldDescriptor(
      name='name', full_name='sae.RegisterRequest.name', index=0,
      number=1, type=9, cpp_type=9, label=2,
      default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],  # TODO(robinson): Implement.
  enum_types=[
  ],
  options=None)


_REGISTERRESPONSE = descriptor.Descriptor(
  name='RegisterResponse',
  full_name='sae.RegisterResponse',
  filename='sae.proto',
  containing_type=None,
  fields=[
    descriptor.FieldDescriptor(
      name='nonce', full_name='sae.RegisterResponse.nonce', index=0,
      number=1, type=9, cpp_type=9, label=2,
      default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],  # TODO(robinson): Implement.
  enum_types=[
  ],
  options=None)


_AUTHREQUEST = descriptor.Descriptor(
  name='AuthRequest',
  full_name='sae.AuthRequest',
  filename='sae.proto',
  containing_type=None,
  fields=[
    descriptor.FieldDescriptor(
      name='name', full_name='sae.AuthRequest.name', index=0,
      number=1, type=9, cpp_type=9, label=2,
      default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='digest', full_name='sae.AuthRequest.digest', index=1,
      number=2, type=9, cpp_type=9, label=2,
      default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],  # TODO(robinson): Implement.
  enum_types=[
  ],
  options=None)


_AUTHRESPONSE = descriptor.Descriptor(
  name='AuthResponse',
  full_name='sae.AuthResponse',
  filename='sae.proto',
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],  # TODO(robinson): Implement.
  enum_types=[
  ],
  options=None)


_SCRIPTREQUEST_KWARG = descriptor.Descriptor(
  name='KWArg',
  full_name='sae.ScriptRequest.KWArg',
  filename='sae.proto',
  containing_type=None,
  fields=[
    descriptor.FieldDescriptor(
      name='key', full_name='sae.ScriptRequest.KWArg.key', index=0,
      number=1, type=9, cpp_type=9, label=2,
      default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='value', full_name='sae.ScriptRequest.KWArg.value', index=1,
      number=2, type=9, cpp_type=9, label=2,
      default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],  # TODO(robinson): Implement.
  enum_types=[
  ],
  options=None)

_SCRIPTREQUEST = descriptor.Descriptor(
  name='ScriptRequest',
  full_name='sae.ScriptRequest',
  filename='sae.proto',
  containing_type=None,
  fields=[
    descriptor.FieldDescriptor(
      name='access_profile', full_name='sae.ScriptRequest.access_profile', index=0,
      number=1, type=11, cpp_type=10, label=2,
      default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='script', full_name='sae.ScriptRequest.script', index=1,
      number=2, type=9, cpp_type=9, label=2,
      default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='kwargs', full_name='sae.ScriptRequest.kwargs', index=2,
      number=3, type=11, cpp_type=10, label=3,
      default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],  # TODO(robinson): Implement.
  enum_types=[
  ],
  options=None)


_SCRIPTRESPONSE = descriptor.Descriptor(
  name='ScriptResponse',
  full_name='sae.ScriptResponse',
  filename='sae.proto',
  containing_type=None,
  fields=[
    descriptor.FieldDescriptor(
      name='result', full_name='sae.ScriptResponse.result', index=0,
      number=1, type=9, cpp_type=9, label=2,
      default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],  # TODO(robinson): Implement.
  enum_types=[
  ],
  options=None)


_FILECHECKSUM = descriptor.Descriptor(
  name='FileChecksum',
  full_name='sae.FileChecksum',
  filename='sae.proto',
  containing_type=None,
  fields=[
    descriptor.FieldDescriptor(
      name='name', full_name='sae.FileChecksum.name', index=0,
      number=1, type=9, cpp_type=9, label=2,
      default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='hash', full_name='sae.FileChecksum.hash', index=1,
      number=2, type=9, cpp_type=9, label=2,
      default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],  # TODO(robinson): Implement.
  enum_types=[
  ],
  options=None)


_MANIFESTREQUEST = descriptor.Descriptor(
  name='ManifestRequest',
  full_name='sae.ManifestRequest',
  filename='sae.proto',
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],  # TODO(robinson): Implement.
  enum_types=[
  ],
  options=None)


_MANIFESTRESPONSE = descriptor.Descriptor(
  name='ManifestResponse',
  full_name='sae.ManifestResponse',
  filename='sae.proto',
  containing_type=None,
  fields=[
    descriptor.FieldDescriptor(
      name='files', full_name='sae.ManifestResponse.files', index=0,
      number=1, type=11, cpp_type=10, label=3,
      default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],  # TODO(robinson): Implement.
  enum_types=[
  ],
  options=None)


_SOFTWAREUPGRADEREQUEST = descriptor.Descriptor(
  name='SoftwareUpgradeRequest',
  full_name='sae.SoftwareUpgradeRequest',
  filename='sae.proto',
  containing_type=None,
  fields=[
    descriptor.FieldDescriptor(
      name='names', full_name='sae.SoftwareUpgradeRequest.names', index=0,
      number=1, type=9, cpp_type=9, label=3,
      default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],  # TODO(robinson): Implement.
  enum_types=[
  ],
  options=None)


_FILECODE = descriptor.Descriptor(
  name='FileCode',
  full_name='sae.FileCode',
  filename='sae.proto',
  containing_type=None,
  fields=[
    descriptor.FieldDescriptor(
      name='name', full_name='sae.FileCode.name', index=0,
      number=1, type=9, cpp_type=9, label=2,
      default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='code', full_name='sae.FileCode.code', index=1,
      number=2, type=9, cpp_type=9, label=2,
      default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],  # TODO(robinson): Implement.
  enum_types=[
  ],
  options=None)


_SOFTWAREUPGRADERESPONSE = descriptor.Descriptor(
  name='SoftwareUpgradeResponse',
  full_name='sae.SoftwareUpgradeResponse',
  filename='sae.proto',
  containing_type=None,
  fields=[
    descriptor.FieldDescriptor(
      name='codes', full_name='sae.SoftwareUpgradeResponse.codes', index=0,
      number=1, type=11, cpp_type=10, label=3,
      default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],  # TODO(robinson): Implement.
  enum_types=[
  ],
  options=None)


_EVENTFILTERREQUEST = descriptor.Descriptor(
  name='EventFilterRequest',
  full_name='sae.EventFilterRequest',
  filename='sae.proto',
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],  # TODO(robinson): Implement.
  enum_types=[
  ],
  options=None)


_EVENTFILTERRESPONSE = descriptor.Descriptor(
  name='EventFilterResponse',
  full_name='sae.EventFilterResponse',
  filename='sae.proto',
  containing_type=None,
  fields=[
    descriptor.FieldDescriptor(
      name='expire', full_name='sae.EventFilterResponse.expire', index=0,
      number=1, type=5, cpp_type=1, label=2,
      default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='sources', full_name='sae.EventFilterResponse.sources', index=1,
      number=2, type=9, cpp_type=9, label=3,
      default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],  # TODO(robinson): Implement.
  enum_types=[
  ],
  options=None)


_EVENTREQUEST_EVENTBODYITEM = descriptor.Descriptor(
  name='EventBodyItem',
  full_name='sae.EventRequest.EventBodyItem',
  filename='sae.proto',
  containing_type=None,
  fields=[
    descriptor.FieldDescriptor(
      name='key', full_name='sae.EventRequest.EventBodyItem.key', index=0,
      number=1, type=9, cpp_type=9, label=2,
      default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='value', full_name='sae.EventRequest.EventBodyItem.value', index=1,
      number=2, type=9, cpp_type=9, label=2,
      default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],  # TODO(robinson): Implement.
  enum_types=[
  ],
  options=None)

_EVENTREQUEST = descriptor.Descriptor(
  name='EventRequest',
  full_name='sae.EventRequest',
  filename='sae.proto',
  containing_type=None,
  fields=[
    descriptor.FieldDescriptor(
      name='timestamp', full_name='sae.EventRequest.timestamp', index=0,
      number=1, type=5, cpp_type=1, label=2,
      default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='ip', full_name='sae.EventRequest.ip', index=1,
      number=2, type=9, cpp_type=9, label=2,
      default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='body', full_name='sae.EventRequest.body', index=2,
      number=3, type=11, cpp_type=10, label=3,
      default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],  # TODO(robinson): Implement.
  enum_types=[
  ],
  options=None)


_EVENTRESPONSE = descriptor.Descriptor(
  name='EventResponse',
  full_name='sae.EventResponse',
  filename='sae.proto',
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],  # TODO(robinson): Implement.
  enum_types=[
  ],
  options=None)


_PINGCHECKREQUEST = descriptor.Descriptor(
  name='PingCheckRequest',
  full_name='sae.PingCheckRequest',
  filename='sae.proto',
  containing_type=None,
  fields=[
    descriptor.FieldDescriptor(
      name='addresses', full_name='sae.PingCheckRequest.addresses', index=0,
      number=1, type=9, cpp_type=9, label=3,
      default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],  # TODO(robinson): Implement.
  enum_types=[
  ],
  options=None)


_PINGCHECKRESPONSE = descriptor.Descriptor(
  name='PingCheckResponse',
  full_name='sae.PingCheckResponse',
  filename='sae.proto',
  containing_type=None,
  fields=[
    descriptor.FieldDescriptor(
      name='reachable', full_name='sae.PingCheckResponse.reachable', index=0,
      number=1, type=9, cpp_type=9, label=3,
      default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='unreachable', full_name='sae.PingCheckResponse.unreachable', index=1,
      number=2, type=9, cpp_type=9, label=3,
      default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],  # TODO(robinson): Implement.
  enum_types=[
  ],
  options=None)


_MESSAGE.fields_by_name['request'].message_type = _REQUEST
_MESSAGE.fields_by_name['response'].message_type = _RESPONSE
_MESSAGE.fields_by_name['error'].message_type = _ERROR
_ERROR.fields_by_name['code'].enum_type = _ERRORCODE
_ACCESSPROFILE.fields_by_name['scheme'].enum_type = _ACCESSSCHEME
_SCRIPTREQUEST.fields_by_name['access_profile'].message_type = _ACCESSPROFILE
_SCRIPTREQUEST.fields_by_name['kwargs'].message_type = _SCRIPTREQUEST_KWARG
_MANIFESTRESPONSE.fields_by_name['files'].message_type = _FILECHECKSUM
_SOFTWAREUPGRADERESPONSE.fields_by_name['codes'].message_type = _FILECODE
_EVENTREQUEST.fields_by_name['body'].message_type = _EVENTREQUEST_EVENTBODYITEM

class Message(message.Message):
  __metaclass__ = reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _MESSAGE

class Request(message.Message):
  __metaclass__ = reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _REQUEST

class Response(message.Message):
  __metaclass__ = reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _RESPONSE

class Error(message.Message):
  __metaclass__ = reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _ERROR

class AccessProfile(message.Message):
  __metaclass__ = reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _ACCESSPROFILE

class PingRequest(message.Message):
  __metaclass__ = reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _PINGREQUEST

class PingResponse(message.Message):
  __metaclass__ = reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _PINGRESPONSE

class RegisterRequest(message.Message):
  __metaclass__ = reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _REGISTERREQUEST

class RegisterResponse(message.Message):
  __metaclass__ = reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _REGISTERRESPONSE

class AuthRequest(message.Message):
  __metaclass__ = reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _AUTHREQUEST

class AuthResponse(message.Message):
  __metaclass__ = reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _AUTHRESPONSE

class ScriptRequest(message.Message):
  __metaclass__ = reflection.GeneratedProtocolMessageType
  
  class KWArg(message.Message):
    __metaclass__ = reflection.GeneratedProtocolMessageType
    DESCRIPTOR = _SCRIPTREQUEST_KWARG
  DESCRIPTOR = _SCRIPTREQUEST

class ScriptResponse(message.Message):
  __metaclass__ = reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _SCRIPTRESPONSE

class FileChecksum(message.Message):
  __metaclass__ = reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _FILECHECKSUM

class ManifestRequest(message.Message):
  __metaclass__ = reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _MANIFESTREQUEST

class ManifestResponse(message.Message):
  __metaclass__ = reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _MANIFESTRESPONSE

class SoftwareUpgradeRequest(message.Message):
  __metaclass__ = reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _SOFTWAREUPGRADEREQUEST

class FileCode(message.Message):
  __metaclass__ = reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _FILECODE

class SoftwareUpgradeResponse(message.Message):
  __metaclass__ = reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _SOFTWAREUPGRADERESPONSE

class EventFilterRequest(message.Message):
  __metaclass__ = reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _EVENTFILTERREQUEST

class EventFilterResponse(message.Message):
  __metaclass__ = reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _EVENTFILTERRESPONSE

class EventRequest(message.Message):
  __metaclass__ = reflection.GeneratedProtocolMessageType
  
  class EventBodyItem(message.Message):
    __metaclass__ = reflection.GeneratedProtocolMessageType
    DESCRIPTOR = _EVENTREQUEST_EVENTBODYITEM
  DESCRIPTOR = _EVENTREQUEST

class EventResponse(message.Message):
  __metaclass__ = reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _EVENTRESPONSE

class PingCheckRequest(message.Message):
  __metaclass__ = reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _PINGCHECKREQUEST

class PingCheckResponse(message.Message):
  __metaclass__ = reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _PINGCHECKRESPONSE


_SAESERVICE = descriptor.ServiceDescriptor(
  name='SAEService',
  full_name='sae.SAEService',
  index=0,
  options=None,
  methods=[
  descriptor.MethodDescriptor(
    name='ping',
    full_name='sae.SAEService.ping',
    index=0,
    containing_service=None,
    input_type=_PINGREQUEST,
    output_type=_PINGRESPONSE,
    options=None,
  ),
  descriptor.MethodDescriptor(
    name='register',
    full_name='sae.SAEService.register',
    index=1,
    containing_service=None,
    input_type=_REGISTERREQUEST,
    output_type=_REGISTERRESPONSE,
    options=None,
  ),
  descriptor.MethodDescriptor(
    name='auth',
    full_name='sae.SAEService.auth',
    index=2,
    containing_service=None,
    input_type=_AUTHREQUEST,
    output_type=_AUTHRESPONSE,
    options=None,
  ),
  descriptor.MethodDescriptor(
    name='manifest',
    full_name='sae.SAEService.manifest',
    index=3,
    containing_service=None,
    input_type=_MANIFESTREQUEST,
    output_type=_MANIFESTRESPONSE,
    options=None,
  ),
  descriptor.MethodDescriptor(
    name='software_upgrade',
    full_name='sae.SAEService.software_upgrade',
    index=4,
    containing_service=None,
    input_type=_SOFTWAREUPGRADEREQUEST,
    output_type=_SOFTWAREUPGRADERESPONSE,
    options=None,
  ),
  descriptor.MethodDescriptor(
    name='script',
    full_name='sae.SAEService.script',
    index=5,
    containing_service=None,
    input_type=_SCRIPTREQUEST,
    output_type=_SCRIPTRESPONSE,
    options=None,
  ),
  descriptor.MethodDescriptor(
    name='event_filter',
    full_name='sae.SAEService.event_filter',
    index=6,
    containing_service=None,
    input_type=_EVENTFILTERREQUEST,
    output_type=_EVENTFILTERRESPONSE,
    options=None,
  ),
  descriptor.MethodDescriptor(
    name='event',
    full_name='sae.SAEService.event',
    index=7,
    containing_service=None,
    input_type=_EVENTREQUEST,
    output_type=_EVENTRESPONSE,
    options=None,
  ),
  descriptor.MethodDescriptor(
    name='ping_check',
    full_name='sae.SAEService.ping_check',
    index=8,
    containing_service=None,
    input_type=_PINGCHECKREQUEST,
    output_type=_PINGCHECKRESPONSE,
    options=None,
  ),
])

class SAEService(service.Service):
  __metaclass__ = service_reflection.GeneratedServiceType
  DESCRIPTOR = _SAESERVICE
class SAEService_Stub(SAEService):
  __metaclass__ = service_reflection.GeneratedServiceStubType
  DESCRIPTOR = _SAESERVICE





from south.db import db
from noc.sa.models import *

class Migration:
    
    def forwards(self):
        db.execute("UPDATE sa_managedobject SET profile_name='DLink.DES3xxx' WHERE profile_name='DLink.DES35xx'")
    
    def backwards(self):
        db.execute("UPDATE sa_managedobject SET profile_name='DLink.DES35xx' WHERE profile_name='DLink.DES3xxx'")



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Django URL dispatcher for module MAIN
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from django.conf.urls.defaults import *
from django.contrib.auth.decorators import login_required
from noc.main.views import index,logout,report,report_index,success,failure,menu,search

urlpatterns = patterns ( None,
        (r"^$",        login_required(index)),
        (r"^logout/$", login_required(logout)),
        (r"^report/(?P<report>[a-z0-9\-_.]+)/$", login_required(report)),
        (r"^report/$", login_required(report_index)),
        (r"success/$", login_required(success)),
        (r"failure/$", login_required(failure)),
        (r"menu/$",    login_required(menu)),
        (r"search/$",  login_required(search)),
)




from south.db import db
from noc.sa.models import *

class Migration:
    
    def forwards(self):
        db.add_column("sa_managedobject","location",models.CharField("Location",max_length=256,null=True,blank=True))
    
    def backwards(self):
        db.delete_column("sa_managedobject","location")



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Macros Framework
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
from noc.lib.registry import Registry
import types,re
##
## Macro Registry
##
class MacroRegistry(Registry):
    name="MacroRegistry"
    subdir="parsers/macros"
    classname="Macro"
    apps=["noc.kb"]
macro_registry=MacroRegistry()

##
## Metaclass for Macroses
##
class MacroBase(type):
    def __new__(cls,name,bases,attrs):
        m=type.__new__(cls,name,bases,attrs)
        macro_registry.register(m.name,m)
        return m
##
## Args regular expression
##
rx_args=re.compile(r"\s*(?P<attr>\S+)\s*=\s*(?P<quote>['\"])(?P<value>.*?)(?P=quote)")
##
## Macro Base
##
class Macro(object):
    __metaclass__=MacroBase
    name=None
    ##
    ## Converts a string of html-like attributes to hash
    ##
    @classmethod
    def parse_args(cls,args):
        if type(args)==types.DictType:
            return args
        return dict([(m[0],m[2]) for m in rx_args.findall(args)])
    ##
    ## Decodes args and calls handle method
    ##
    @classmethod
    def expand(cls,args,text):
        return cls.handle(cls.parse_args(args),text)
    ##
    ## Specific macro handler to be overriden in child classes
    ## Accepts a hash of args and text and returns formatted HTML
    ## to be included in output
    ##
    @classmethod
    def handle(cls,args,text):
        raise NotImplementedError




# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
# Force10.FTOS.get_vlans
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
import noc.sa.script
from noc.sa.interfaces import IGetVlans
from noc.lib.text import parse_table
import re

class Script(noc.sa.script.Script):
    name="Force10.FTOS.get_vlans"
    implements=[IGetVlans]
    def execute(self):
        self.cli("terminal length 0")
        return [{"vlan_id":x[0],"name":x[1]} for x in parse_table(self.cli("show vlan brief"))]



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Search engine with plugin support
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from django.db.models.signals import class_prepared
##
## A hash of Model.search classmethods.
## Populated by "class_prepared" signal listener
## Model.search is a generator taking parameters (user,query,limit)
## And yielding a SearchResults (ordered by relevancy)
##
search_methods={}

##
## Register new search handler if model has .search classmethod
##
def on_new_model(sender,**kwargs):
    if hasattr(sender,"search"):
        search_methods[getattr(sender,"search")]=None

##
## Attach to the 'class_prepared' signal
## and on_new_model on every new model
##
class_prepared.connect(on_new_model)

##
## Wrapper for search results
##
class SearchResult(object):
    def __init__(self,url,title,text,relevancy=1.0):
        self.url=url
        self.title=title
        self.text=text
        self.relevancy=relevancy
    
    def __cmp__(self,other):
        return cmp(other.relevancy,self.relevancy)
##
## Interface to built-in search system
##
def search(user,query,limit=100):
    r=[]
    for sm in search_methods:
        r+=list(sm(user,query,limit))
    return sorted(r)[:limit]




from south.db import db
from noc.main.models import *

class Migration:
    
    def forwards(self):
        
        # Model 'DatabaseStorage'
        db.create_table('main_databasestorage', (
            ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
            ('name', models.CharField("Name",max_length=256,unique=True)),
            ('data', BinaryField("Data")),
            ('size', models.IntegerField("Size")),
            ('mtime', models.DateTimeField("MTime"))
        ))
        
        db.send_create_signal('main', ['DatabaseStorage'])
    
    def backwards(self):
        db.delete_table('main_databasestorage')
        



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Correlator daemon
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from __future__ import with_statement
from noc.lib.daemon import Daemon
from noc.fm.models import Event,EventCorrelationRule
from noc.lib.fileutils import rewrite_when_differ
from pyke import knowledge_engine
import logging,time,os

##
## Default knowledge base name
##
KNOWLEDGE_BASE="kb"
##
## noc-correlator daemon
##
class Correlator(Daemon):
    daemon_name="noc-correlator"
    def __init__(self):
        Daemon.__init__(self)
        logging.info("Running Correlator")
    ##
    ## Compile and build rule base
    ##
    def build_rulebase(self):
        s=[]
        for r in EventCorrelationRule.objects.order_by("name"):
            s.append(r.pyke_code)
        c=["local","fm","rules","correlation"]
        d=os.path.join(*c)
        try:
            os.makedirs(d)
        except:
            pass
        # Write krb
        rewrite_when_differ(os.path.join(d,"%s.krb"%KNOWLEDGE_BASE),"\n".join(s))
        # Write modules __init__.py files when necessary
        for i in range(len(c)):
            path=os.path.join(*c[:i+1]+["__init__.py"])
            if not os.path.exists(path):
                open(path,"w").close()
    
    ##
    ## Populate knowledge base with facts
    ##
    def load_window(self):
        r=[]
        self.ke.reset()
        # Load event window.
        # Populate knowledge base by facts
        for e in Event.objects.order_by("-timestamp")[:140]:
            en=e.id
            self.ke.assert_("fm","event_class",(en,str(e.event_class.name)))
            self.ke.assert_("fm","managed_object",(en,str(e.managed_object.name)))
            self.ke.assert_("fm","timestamp",(en,int(time.mktime(e.timestamp.timetuple()))))
            for v in e.eventdata_set.filter(type="V"):
                self.ke.assert_("fm","var",(en,str(v.key),str(v.value)))
            r.append(e)
        # Activate rulebase again
        self.ke.activate(KNOWLEDGE_BASE)
        return r
    ##
    ## Search possible solutions for rule(event,$x)
    ##
    def search_1(self,rule,event):
        with self.ke.prove_n(KNOWLEDGE_BASE,rule,(event,),1) as gen:
            for ans in gen:
                yield ans[0][0]
    ##
    ## Search possible solutions for rule($x,$y)
    ##
    def search_2(self,rule):
        with self.ke.prove_n(KNOWLEDGE_BASE,rule,tuple(),2) as gen:
            for ans in gen:
                yield ans[0]
    ##
    ## 
    ##
    def start_trace(self):
        for r in self.ke.get_rb(KNOWLEDGE_BASE).rules.keys():
            self.ke.trace(KNOWLEDGE_BASE,r)
    ##
    ## main daemon loop
    ##
    def run(self):
        self.build_rulebase()
        self.ke=knowledge_engine.engine("noc.local.fm.rules.correlation")
        events=self.load_window()
        #self.ke.get_kb("fm").dump_specific_facts()
        #self.start_trace()
        t0=time.time()
        n=0
        for x,y in self.search_2("close"):
            print x,y
            n+=1
        logging.debug("%d closing pairs found"%n)
        dt=time.time()-t0
        ne=len(events)
        if dt>0:
            p=ne/dt
        else:
            p=len(events)
        logging.debug("%d events processed in %8.2f seconds (%8.2f events/sec)"%(ne,dt,p))
        self.ke.print_stats()



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Cisco.IOS miscellaneous rules
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from noc.fm.rules.classification import ClassificationRule,DROP_EVENT
from noc.fm.rules.classes.default import DROP
##
## Cisco.IOS TCP Connection Closed
##
class Cisco_IOS_TCP_Connection_Closed_Rule(ClassificationRule):
    name="Cisco.IOS TCP Connection Closed"
    event_class=DROP
    preference=90000
    action=DROP_EVENT
    required_mibs=["CISCOTRAP-MIB"]
    patterns=[
        (r"^source$",r"^SNMP Trap$"),
        (r"^1\.3\.6\.1\.6\.3\.1\.1\.4\.1\.0$",r"^1\.3\.6\.1\.4\.1\.9\.0\.1$"),
        (r"^profile$",r"^Cisco\.IOS$"),
    ]




from south.db import db
from noc.kb.models import *

class Migration:
    depends_on=(
        ("main","0004_language"),
    )
    def forwards(self):
        
        # Model 'KBCategory'
        db.create_table('kb_kbcategory', (
            ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
            ('name', models.CharField("Name",max_length=64,unique=True))
        ))
        
        # Mock Models
        Language = db.mock_model(model_name='Language', db_table='main_language', db_tablespace='', pk_field_name='id', pk_field_type=models.AutoField)
        
        # Model 'KBEntry'
        db.create_table('kb_kbentry', (
            ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
            ('subject', models.CharField("Subject",max_length=256)),
            ('body', models.TextField("Body")),
            ('language', models.ForeignKey(Language,verbose_name=Language)),
            ('markup_language', models.CharField("Markup Language",max_length="16"))
        ))
        # Mock Models
        KBEntry = db.mock_model(model_name='KBEntry', db_table='kb_kbentry', db_tablespace='', pk_field_name='id', pk_field_type=models.AutoField)
        KBCategory = db.mock_model(model_name='KBCategory', db_table='kb_kbcategory', db_tablespace='', pk_field_name='id', pk_field_type=models.AutoField)
        
        # M2M field 'KBEntry.categories'
        db.create_table('kb_kbentry_categories', (
            ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
            ('kbentry', models.ForeignKey(KBEntry, null=False)),
            ('kbcategory', models.ForeignKey(KBCategory, null=False))
        )) 
        
        # Mock Models
        KBEntry = db.mock_model(model_name='KBEntry', db_table='kb_kbentry', db_tablespace='', pk_field_name='id', pk_field_type=models.AutoField)
        User = db.mock_model(model_name='User', db_table='auth_user', db_tablespace='', pk_field_name='id', pk_field_type=models.AutoField)
        
        # Model 'KBEntryHistory'
        db.create_table('kb_kbentryhistory', (
            ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
            ('kb_entry', models.ForeignKey(KBEntry,verbose_name="KB Entry")),
            ('timestamp', models.DateTimeField("Timestamp",auto_now_add=True)),
            ('user', models.ForeignKey(User,verbose_name=User)),
            ('diff', models.TextField("Diff"))
        ))
        
        db.send_create_signal('kb', ['KBCategory','KBEntry','KBEntryHistory'])
    
    def backwards(self):
        db.delete_table('kb_kbentryhistory')
        db.delete_table('kb_kbentry_categories')
        db.delete_table('kb_kbcategory')
        db.delete_table('kb_kbentry')



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""

from noc.main.report import Column
import noc.main.report

class Report(noc.main.report.Report):
    name="dns.duplicated_fqdns"
    title="Duplicated FQDNs"
    requires_cursor=True
    columns=[Column("FQDN"),Column("IPs")]
    
    def get_queryset(self):
        vrf_id=self.execute("SELECT id FROM ip_vrf WHERE rd='0:0'")[0][0]
        r=[]
        for fqdn,c in self.execute("""
                SELECT fqdn,COUNT(*)
                FROM ip_ipv4address
                WHERE vrf_id=%s
                GROUP BY 1
                HAVING COUNT(*)>1
                ORDER BY 2 DESC""",[vrf_id]):
            ips=", ".join([x[0] for x in self.execute("SELECT ip FROM ip_ipv4address WHERE fqdn=%s and vrf_id=%s",[fqdn,vrf_id])])
            r+=[[fqdn,ips]]
        return r



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from noc.main.report import Column
import noc.main.report
from noc.sa.models import script_registry

class Report(noc.main.report.Report):
    name="sa.scripts"
    title="Scripts"
    requires_cursor=False
    columns=[Column("Script"),Column("Interfaces")]
    
    def get_queryset(self):
        def get_interfaces(c):
            return ", ".join([i.__class__.__name__ for i in c.implements])
        return sorted([[n,get_interfaces(c)] for n,c in script_registry.classes.items()])



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
import noc.sa.script
from noc.sa.interfaces import IGetVlans

class Script(noc.sa.script.Script):
    name="Alcatel.AOS.get_vlans"
    implements=[IGetVlans]
    def execute(self):
        vlans=self.cli("show vlan")
        r=[]
        in_body=False
        for l in vlans.split("\n"):
            l=l.strip()
            if not in_body:
                if l.startswith("----"):
                    in_body=True
                continue
            try:
                vlan_id,name,ports,vlan_type,auth=l.split()
            except:
                continue
            r.append({"vlan_id":int(vlan_id),"name":name})
        return r



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Small DSL for event class description
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from noc.fm.models import EventCategory,EventPriority,EventClassVar,event_trigger_registry
import noc.fm.models
import re

rx_var=re.compile("{{([^}]+)}}")
##
## EventCategory cache
##
category_cache={}
def get_category(name):
    global category_cache
    if name not in category_cache:
        try:
            category_cache[name]=EventCategory.objects.get(name=name)
        except EventCategory.DoesNotExist:
            category_cache[name]=EventCategory(name=name)
            category_cache[name].save()
    return category_cache[name]
##
## EventPriority cache
##
priority_cache={}
def get_priority(name):
    global priority_cache
    if name not in priority_cache:
        priority_cache[name]=EventPriority.objects.get(name=name)
    return priority_cache[name]
    
##
## Event class variable placeholder.
## Used inside class Vars of Event Class
##
class Var(object):
    def __init__(self,required=True,repeat=False):
        self.required=required
        self.repeat=repeat
##
## Metaclass performing registration
## and variable checking
##
class EventClassBase(type):
    def __new__(cls,name,bases,attrs):
        def get_vars(s):
            vars=rx_var.findall(s)
            if vars:
                return dict(zip(vars,vars))
            else:
                return {}
        m=type.__new__(cls,name,bases,attrs)
        #
        # Extract vars from Vars class
        #
        m.vars={}
        for vn in [v for v in dir(m.Vars)]:
            v=getattr(m.Vars,vn)
            if isinstance(v,Var):
                m.vars[vn]=v
        #
        # Check templates contain no undefined variables
        #
        tv={}
        tv.update(get_vars(m.subject_template))
        tv.update(get_vars(m.body_template))
        for v in tv:
            if v not in m.vars:
                raise Exception("Unknown variable '%s' in template"%v)
        #
        return m
##
## Base class for event class descriptions
##
class EventClass(object):
    __metaclass__=EventClassBase
    name="Event Class"
    category="DEFAULT"
    priority="DEFAULT"
    subject_template=""
    body_template=""
    repeat_suppression=False
    repeat_suppression_interval=3600
    trigger=None
    class Vars: pass
    ##
    ## Syncronize class with database
    ##
    @classmethod
    def sync(cls):
        # Check trigger
        if cls.trigger and cls.trigger not in event_trigger_registry.classes:
            raise "Invalid event trigger: %s"%cls.trigger
        # Create/update event class
        try:
            ec=noc.fm.models.EventClass.objects.get(name=cls.name)
            ec.category=get_category(cls.category)
            ec.default_priority=get_priority(cls.priority)
            ec.subject_template=cls.subject_template
            ec.body_template=cls.body_template
            ec.repeat_suppression=cls.repeat_suppression
            ec.repeat_suppression_interval=cls.repeat_suppression_interval
            ec.trigger=cls.trigger
            ec.is_builtin=True
            print "UPDATE CLASS %s"%cls.name
        except noc.fm.models.EventClass.DoesNotExist:
            ec=noc.fm.models.EventClass(
                name=cls.name,
                category=get_category(cls.category),
                default_priority=get_priority(cls.priority),
                subject_template=cls.subject_template,
                body_template=cls.body_template,
                repeat_suppression=cls.repeat_suppression,
                repeat_suppression_interval=cls.repeat_suppression_interval,
                trigger=cls.trigger,
                is_builtin=True)
            print "CREATE CLASS %s"%cls.name
        ec.save()
        # Syncronize vars
        ec.eventclassvar_set.all().delete()
        for name,var in cls.vars.items():
            v=EventClassVar(event_class=ec,name=name,required=var.required,repeat_suppression=var.repeat)
            v.save()



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## "now"search"" macro
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
from noc.kb.parsers.macros import Macro as MacroBase
from django.db.models import Q
##
## "search" macro
## USAGE:
## <<search [criteria] [order_by] [limit] [title]>>
## WHERE:
##     criteria:
##         category=cat1,...,catN
##         language=lang
##     order_by - one of "subject","-subject","id","-id"
##     limit
##     title - table title
##     display_list - list of fields ("id","subject")
##
class Macro(MacroBase):
    name="search"
    @classmethod
    def handle(cls,args,text):
        from noc.kb.models import KBEntry
        # Build search criteria
        if "category" in args:
            q=Q()
            for cn in args["category"].split(","):
                cn=cn.strip()
                q&=Q(categories__name=cn)
        else:
            q=Q()
        if "language" in args:
            q&=Q(language__name=args["language"])
        q=KBEntry.objects.filter(q) # Flatten to query
        # Apply ordering
        if "order_by" in args:
            if args["order_by"] not in ["subject","-subject","id","-id"]:
                raise Exception("Invalid order_by value")
            q=q.order_by(args["order_by"])
        # Apply limition
        if "limit" in args:
            q=q[:int(args["limit"])]
        # Build display list
        if "display_list" in args:
            display_list=[]
            for f in args["display_list"].split(","):
                f=f.strip()
                if f not in ("id","subject"):
                    raise Exception("Invalid field %s"%f)
                display_list.append(f)
        else:
            display_list=["id","subject"]
        # Render
        out=["<table border='0'>"]
        if "title" in args:
            out+=["<tr><th>%s</th></tr>"%args["title"]]
        for a in q:
            link="/kb/%d/"%a.id
            out+=["<tr>"]
            for f in display_list:
                out+=["<td><a href='%s'>%s</a></td>"%(link,getattr(a,f))]
            out+=["</tr>"]
        out+=["</table>"]
        return u"\n".join(out)



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Object resolver supervisor
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from noc.peer.whois import Whois
from noc.lib.validators import is_asn,is_cidr
from noc.lib.nbsocket import SocketFactory
import sets,logging

class AS(object):
    def __init__(self,parent,name):
        self.parent=parent
        self.name=name
        self.prefixes=sets.Set([])
        self.parent.ases[self.name]=self
        self.as_sets=sets.Set([])
        self.is_new=True
        logging.debug("Creating %s"%self.name)
        
    def __repr__(self):
        return self.name
        
    def resolve(self):
        if self.is_new:
            logging.debug("Resolving %s"%self)
            self.parent.schedule_whois("-i origin %s"%self.name,self.add_prefixes,["route"])
            self.is_new=False
        
    def add_prefixes(self,prefixes):
        logging.debug("Add %s to %s"%(prefixes,self.name))
        for k,v in prefixes:
            if is_cidr(v):
                self.prefixes.add(v)
        for as_set in self.as_sets:
            as_set.add_prefixes(self.prefixes)
                
    def attach_to_asset(self,as_set):
        as_set.add_prefixes(self.prefixes)
        self.as_sets.add(as_set)
        
                
class ASSet(object):
    def __init__(self,parent,name):
        self.parent=parent
        self.name=name
        self.members=sets.Set([])
        self.parent.as_sets[self.name]=self
        self.prefixes=sets.Set([])
        self.as_sets=sets.Set([])
        self.is_new=True
        logging.debug("Creating %s"%self.name)
        
    def __repr__(self):
        return self.name
        
    def resolve(self):
        if not self.is_new:
            return
        logging.debug("Resolving %s"%self)
        self.is_new=False
        if is_asn(self.name[2:]):
            a=self.parent.create_as(self.name)
            self.add_member(a)
            a.attach_to_asset(self)
            a.resolve()
        else:
            self.parent.schedule_whois("%s"%self.name,self.add_members,["members"])
            
    def add_members(self,members):
        logging.debug("Add %s to %s"%(str(members),self))
        ms=sets.Set([])
        for k,v in members:
            ms.update([x.strip().upper() for x in v.split(",")])
        for m in ms:
            if is_asn(m):
                a=self.parent.create_as(m)
                self.add_member(a)
                a.attach_to_asset(self)
                a.resolve()
            else:
                a=self.parent.create_as_set(m)
                a.attach_to_asset(self)
                a.resolve()
                    
    def add_prefixes(self,prefixes):
        logging.debug("Add %s to %s"%(prefixes,self.name))
        if not prefixes.issubset(self.prefixes):
            self.prefixes.update(prefixes)
            for as_set in self.as_sets:
                as_set.add_prefixes(prefixes)
            
    def attach_to_asset(self,as_set):
        logging.debug("Attaching %s to %s"%(self,as_set))
        self.as_sets.add(as_set)
        as_set.add_prefixes(self.prefixes)
        as_set.members.update(self.members)
        
    def add_member(self,a):
        if a not in self.members:
            self.members.add(a)
            for as_set in self.as_sets:
                as_set.add_member(a)
            

class Resolver(object):
    def __init__(self,whois_concurrency=5):
        self.whois_queue=[]
        self.ases={}
        self.as_sets={}
        self.whois_concurrency=whois_concurrency
        
    def create_as(self,name):
        if name in self.ases:
            return self.ases[name]
        else:
            return AS(self,name)
            
    def create_as_set(self,name):
        if name in self.as_sets:
            return self.as_sets[name]
        else:
            return ASSet(self,name)
        
    def schedule_whois(self,query,callback,fields=None):
        logging.debug("Scheduliung whois query: %s"%query)
        self.whois_queue.append((query,callback,fields))
    #
    # Resolves a list of as-sets and returns a hash of
    # name -> (members,prefixes)
    #
    def resolve(self,as_sets):
        factory=SocketFactory()
        for x in as_sets:
            if not x in self.ases:
                a=ASSet(self,x)
                a.resolve()
        while self.whois_queue or len(factory):
            while self.whois_queue and len(factory)<self.whois_concurrency:
                q,c,f=self.whois_queue.pop(0)
                Whois(factory,q,c,f)
            factory.loop()
        r={}
        for n,a in self.as_sets.items():
            r[n]=(a.members,a.prefixes)
        return r

def resolve(as_sets):
    logging.basicConfig(level=logging.DEBUG)
    r=Resolver()
    return r.resolve(as_sets)



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## URL Processing functions
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
import re,urllib

rx_url=re.compile(r"^(?P<scheme>[^:]+)://(?:(?P<user>[^:]+):(?P<password>[^@]+)@)(?P<host>[^/:]+)(?::(?P<port>\d+))?(?P<path>.*)$")

class InvalidURLException(Exception): pass

class URL(object):
    def __init__(self,url):
        self.url=url
        match=rx_url.match(self.url)
        if not match:
            raise InvalidURLException
        self.scheme=match.group("scheme")
        self.user=urllib.unquote(match.group("user"))
        self.password=urllib.unquote(match.group("password"))
        self.host=match.group("host")
        if match.group("port"):
            self.port=int(match.group("port"))
        else:
            self.port=None
        self.path=match.group("path")



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Import MoinMoin wiki data to NOC KB
## USAGE:
## python manage.py convert-moin [--encoding=charset] [--language=lang] [--category=category] <path to moin data/ >
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from __future__ import with_statement
from django.core.management.base import BaseCommand
from django.core.files.base import ContentFile
from django.db import transaction,reset_queries
from django.contrib.auth.models import User
from noc.main.models import Language,DatabaseStorage,database_storage
from noc.kb.models import KBEntry,KBEntryAttachment,KBCategory
from optparse import OptionParser, make_option
import os,re,stat,datetime,sys,types,gc

##
rx_hexseq=re.compile(r"\(((?:[0-9a-f][0-9a-f])+)\)")
##
## convert-moin command handler
##
class Command(BaseCommand):
    help="Import MoinMoin wiki into NOC KB"
    option_list=BaseCommand.option_list+(
        make_option("-e","--encoding",dest="encoding",default="utf-8"),
        make_option("-l","--language",dest="language",default="English"),
        make_option("-c","--category",dest="category")
    )
    def handle(self, *args, **options):
        self.encoding=options["encoding"]
        self.pages=os.path.join(args[0],"pages")
        transaction.enter_transaction_management()
        self.user=User.objects.order_by("id")[0] # Get first created user as owner
        self.language=Language.objects.get(name=options["language"])
        # Find category
        self.category=None
        if options["category"]:
            try:
                self.category=KBCategory.objects.get(name=options["category"])
            except KBCategory.DoesNotExist:
                self.category=KBCategory(name=options["category"])
                self.category.save()
        oc=len(gc.get_objects())
        for page in os.listdir(self.pages):
            self.convert_page(page)
            reset_queries()
            gc.collect()
            new_oc=len(gc.get_objects())
            self.out("%d leaked objects\n"%(new_oc-oc))
            oc=new_oc
        transaction.commit()
        transaction.leave_transaction_management()
    ##
    ## Progress output
    ##
    def out(self,s):
        if type(s)==types.UnicodeType:
            sys.stdout.write(s.encode("utf-8"))
        else:
            sys.stdout.write(unicode(s,self.encoding).encode("utf-8"))
        sys.stdout.flush()
    ##
    ## Convert single MoinMoin page
    ##
    def convert_page(self,page):
        # Convert (hex) sequences to unicode
        def convert_hexseq(m):
            seq=m.group(1)
            r=[]
            while seq:
                c=seq[:2]
                seq=seq[2:]
                r+=chr(int(c,16))
            r="".join(r)
            return unicode(r,self.encoding)
        root=os.path.join(self.pages,page)
        name=rx_hexseq.sub(convert_hexseq,page)
        self.out("Converting %s (%s)..."%(page,name))
        # Find current revisions
        current_path=os.path.join(root,"current")
        if not os.path.exists(current_path):
            return  # Return on incomplete pages
        with open(current_path) as f:
            current=f.read().split()
        # Write all revisions
        kbe=None
        revisions=sorted(os.listdir(os.path.join(root,"revisions")))
        for rev in revisions:
            rev_path=os.path.join(root,"revisions",rev)
            with open(rev_path) as f:
                body=self.convert_body(unicode(f.read(),self.encoding))
            mtime=datetime.datetime.fromtimestamp(os.stat(rev_path)[stat.ST_MTIME]) # Revision time
            if kbe is None:
                kbe=KBEntry(subject=name,body=body,language=self.language,markup_language="Creole")
                kbe.save(user=self.user,timestamp=mtime) # Revision history will be populated automatically
                if self.category:
                    kbe.categories.add(self.category)
            else:
                kbe.body=body
                kbe.save(user=self.user,timestamp=mtime) # Revision history will be populated automatically
        self.out("... %d revisions\n"%len(revisions))
        if kbe is None:
            return # Return when no revisions found
        # Write all attachments
        attachments_root=os.path.join(root,"attachments")
        if os.path.isdir(attachments_root):
            for a in os.listdir(attachments_root):
                self.out("     %s..."%a)
                a_path=os.path.join(attachments_root,a)
                mtime=datetime.datetime.fromtimestamp(os.stat(a_path)[stat.ST_MTIME]) # Attach modification time
                with open(a_path) as f:
                    dbs_path="/kb/%d/%s"%(kbe.id,a)
                    database_storage.save(dbs_path,f)
                    # Correct mtime
                    database_storage.set_mtime(dbs_path,mtime)
                KBEntryAttachment(kb_entry=kbe,name=a,file=dbs_path).save()
                self.out("...done\n")
    ##
    ## Convert MoinMoin syntax to Creole
    ##
    def convert_body(self,body):
        return body



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from south.db import db
from noc.cm.models import *

class Migration:
    
    def forwards(self):
        
        # Model 'ObjectCategory'
        db.create_table('cm_objectcategory', (
            ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
            ('name', models.CharField("Name",max_length=64,unique=True)),
            ('description', models.CharField("Description",max_length=128,null=True,blank=True))
        ))
        # Model 'Object'
        db.create_table('cm_object', (
            ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
            ('handler_class_name', models.CharField("Object Type",max_length=64)),
            ('stream_url', models.CharField("URL",max_length=128)),
            ('profile_name', models.CharField("Profile",max_length=128)),
            ('repo_path', models.CharField("Repo Path",max_length=128)),
            ('push_every', models.PositiveIntegerField("Push Every (secs)",default=86400,blank=True,null=True)),
            ('next_push', models.DateTimeField("Next Push",blank=True,null=True)),
            ('last_push', models.DateTimeField("Last Push",blank=True,null=True)),
            ('pull_every', models.PositiveIntegerField("Pull Every (secs)",default=86400,blank=True,null=True)),
            ('next_pull', models.DateTimeField("Next Pull",blank=True,null=True)),
            ('last_pull', models.DateTimeField("Last Pull",blank=True,null=True))
        ))
        # Mock Models
        Object = db.mock_model(model_name='Object', db_table='cm_object', db_tablespace='', pk_field_name='id', pk_field_type=models.AutoField)
        ObjectCategory = db.mock_model(model_name='ObjectCategory', db_table='cm_objectcategory', db_tablespace='', pk_field_name='id', pk_field_type=models.AutoField)
        
        # M2M field 'Object.categories'
        db.create_table('cm_object_categories', (
            ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
            ('object', models.ForeignKey(Object, null=False)),
            ('objectcategory', models.ForeignKey(ObjectCategory, null=False))
        )) 
        db.create_index('cm_object', ['handler_class_name','repo_path'], unique=True, db_tablespace='')
        
        
        db.send_create_signal('cm', ['ObjectCategory','Object'])
    
    def backwards(self):
        db.delete_table('cm_object_categories')
        db.delete_table('cm_object')
        db.delete_table('cm_objectcategory')



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Rebuild online documentation
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from __future__ import with_statement
from django.core.management.base import BaseCommand
import os,glob,subprocess,csv,cStringIO
from noc.lib.fileutils import rewrite_when_differ

##
## Command handler
##
class Command(BaseCommand):
    help="Syncronize online documentation"
    ##
    ## Rebuild supported equipment database.
    ## Returns true if database was updated
    ##
    def update_se_db(self):
        out=cStringIO.StringIO()
        writer=csv.writer(out)
        for dirpath,dirname,files in os.walk("sa/profiles/"):
            if "supported.csv" in files:
                pp=dirpath.split(os.path.sep)
                profile="%s.%s"%(pp[-2],pp[-1])
                with open(os.path.join(dirpath,"supported.csv")) as f:
                    r=[]
                    for row in csv.reader(f):
                        if len(row)!=3:
                            continue
                        vendor,model,version=row
                        m="%s %s"%(vendor,model)
                        r+=[(profile,m,version)]
                    for r in sorted(r):
                        writer.writerow(r)
        db_path="local/supported.csv"
        return rewrite_when_differ(db_path,out.getvalue())
        
    def handle(self, *args, **options):
        se_db_updated=self.update_se_db()
        # Find and build all makefiles
        for makefile in glob.glob("share/doc/*/*/Makefile"):
            d,f=os.path.split(makefile)
            env=None
            if se_db_updated:
                env=os.environ.copy()
                env["OPTIONS"]="-a"
            subprocess.call(["make","html"],cwd=d,env=env)



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Vendor: Zebra
## OS:     Zebra
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
import noc.sa.profiles
from noc.sa.protocols.sae_pb2 import TELNET,SSH

class Profile(noc.sa.profiles.Profile):
    name="Zebra.Zebra"
    supported_schemes=[TELNET,SSH]
    pattern_more="^--More-- "
    pattern_unpriveleged_prompt=r"^\S+?>"
    command_super="enable"
    pattern_prompt=r"^\S+?#"
    command_more=" "
    requires_netmask_conversion=True
    
    def generate_prefix_list(self,name,pl,strict=True):
        p="ip prefix-list %s permit %%s"%name
        if not strict:
            p+=" le 32"
        return "no ip prefix-list %s\n"%name+"\n".join([p%x for x in pl])



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## f5.BIGIP.get_version
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
import noc.sa.script
from noc.sa.interfaces import IGetVersion
import re

rx_ver=re.compile("BIG-IP Version (?P<version>.+?)$",re.MULTILINE)

class Script(noc.sa.script.Script):
    name="f5.BIGIP.get_version"
    implements=[IGetVersion]
    def execute(self):
        v=self.cli("version")
        match=rx_ver.search(v)
        return {
            "vendor"    : "f5",
            "platform"  : "BIG-IP",
            "version"   : match.group("version"),
        }



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from south.db import db
from noc.ip.models import *

class Migration:
    depends_on=(
        ("sa","0002_trigger"),
    )
    def forwards(self):
        if not self.has_column("ip_ipv4block","prefix_cidr"):
            db.execute("ALTER TABLE ip_ipv4block ADD prefix_cidr CIDR")
            db.execute("UPDATE ip_ipv4block SET prefix_cidr=prefix::cidr")
            db.execute("ALTER TABLE ip_ipv4block ALTER prefix_cidr SET NOT NULL")
            db.execute("CREATE INDEX x_ip_ipv4block_prefix_cidr ON ip_ipv4block(prefix_cidr)")
        if not self.has_column("ip_ipv4blockaccess","prefix_cidr"):
            db.execute("ALTER TABLE ip_ipv4blockaccess ADD prefix_cidr CIDR")
            db.execute("UPDATE ip_ipv4blockaccess SET prefix_cidr=prefix::cidr")
            db.execute("ALTER TABLE ip_ipv4blockaccess ALTER prefix_cidr SET NOT NULL")
        db.execute(RAW_SQL_CREATE)
        if not self.has_trigger("ip_ipv4block","t_ip_ipv4block_modify"):
            db.execute(t_ip_ipv4block_modify)
        if not self.has_trigger("ip_ipv4blockaccess","t_ip_ipv4blockaccess_modify"):
            db.execute(t_ip_ipv4blockaccess_modify)
        
    def backwards(self):
        db.execute(RAW_SQL_DROP)
        db.execute("ALTER TABLE ip_ipv4block DROP COLUMN prefix_cidr")
        db.execute("ALTER TABLE ip_ipv4blockaccess DROP COLUMN prefix_cidr")
        
    def has_column(self,table,name):
        return db.execute("SELECT COUNT(*)>0 FROM pg_attribute a JOIN pg_class p ON (p.oid=a.attrelid)"\
            +" WHERE p.relname='%s' AND a.attname='%s'"%(table,name))[0][0]
        
    def has_trigger(self,table,name):
        return db.execute("SELECT COUNT(*)>0 FROM pg_trigger t JOIN pg_class p ON (p.oid=t.tgrelid)"\
            +" WHERE p.relname='%s' AND t.tgname='%s'"%(table,name))[0][0]
        
RAW_SQL_CREATE="""
CREATE OR REPLACE
FUNCTION ip_ipv4_block_depth(INTEGER,CIDR,CIDR)
RETURNS INTEGER
AS $$
DECLARE
    vrf         ALIAS FOR $1;
    inner_block ALIAS FOR $2;
    outer_block ALIAS FOR $3;
    c   INTEGER;
BEGIN
    SELECT COUNT(*)
    INTO   c
    FROM ip_ipv4block
    WHERE vrf_id=vrf AND prefix_cidr >> inner_block AND prefix_cidr << outer_block;

    RETURN c;
END;
$$ LANGUAGE plpgsql;

CREATE OR REPLACE
FUNCTION ip_ipv4_block_depth_in_vrf_group(INTEGER,CIDR,CIDR)
RETURNS INTEGER
AS $$
DECLARE
    p_vrf_group_id ALIAS FOR $1;
    inner_block    ALIAS FOR $2;
    outer_block    ALIAS FOR $3;
    c   INTEGER;
BEGIN
    SELECT COUNT(*)
    INTO   c
    FROM   ip_ipv4block b JOIN ip_vrf v ON (b.vrf_id=v.id)
    WHERE  v.vrf_group_id=p_vrf_group_id AND prefix_cidr >> inner_block AND prefix_cidr << outer_block;

    RETURN c;
END;
$$ LANGUAGE plpgsql;

CREATE OR REPLACE
FUNCTION hostname(TEXT)
RETURNS TEXT
AS $$
SELECT SUBSTRING($1 from E'^[a-zA-Z0-9\\-]+');
$$ LANGUAGE SQL IMMUTABLE;

CREATE OR REPLACE
FUNCTION domainname(TEXT)
RETURNS TEXT
AS $$
SELECT SUBSTRING($1 from E'^[a-zA-Z0-9\\-]+\\.(.+)'); 
$$ LANGUAGE SQL IMMUTABLE;

CREATE OR REPLACE
FUNCTION f_trigger_ip_ipv4block()
RETURNS TRIGGER
AS $$
BEGIN
    NEW.prefix_cidr:=NEW.prefix;
    RETURN NEW;
END;
$$ LANGUAGE plpgsql;

CREATE OR REPLACE
FUNCTION f_trigger_ip_ipv4blockaccess()
RETURNS TRIGGER
AS $$
BEGIN
    NEW.prefix_cidr:=NEW.prefix;
    RETURN NEW;
END;
$$ LANGUAGE plpgsql;"""

t_ip_ipv4block_modify="""
CREATE TRIGGER t_ip_ipv4block_modify
BEFORE INSERT OR UPDATE ON ip_ipv4block
FOR EACH ROW EXECUTE PROCEDURE f_trigger_ip_ipv4block();
"""

t_ip_ipv4blockaccess_modify="""
CREATE TRIGGER t_ip_ipv4blockaccess_modify
BEFORE INSERT OR UPDATE ON ip_ipv4blockaccess
FOR EACH ROW EXECUTE PROCEDURE f_trigger_ip_ipv4blockaccess();
"""

RAW_SQL_DROP="""
DROP TRIGGER IF EXISTS t_ip_ipv4block_modify ON ip_ipv4block;
DROP TRIGGER IF EXISTS t_ip_ipv4blockaccess_modify ON ip_ipv4blockaccess;
DROP FUNCTION f_trigger_ip_ipv4block();
DROP FUNCTION f_trigger_ip_ipv4blockaccess();
DROP FUNCTION ip_ipv4_block_depth(INTEGER,CIDR,CIDR);
DROP FUNCTION ip_ipv4_block_depth_in_vrf_group(INTEGER,CIDR,CIDR);
DROP FUNCTION hostname(TEXT);
DROP FUNCTION domainname(TEXT);
"""



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Various system utils
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from __future__ import with_statement
import ConfigParser,os,signal
##
## Read daemon config
##
def get_daemon_config(daemon_name):
    config=ConfigParser.SafeConfigParser()
    config.read("etc/%s.defaults"%daemon_name)
    config.read("etc/%s.conf"%daemon_name)
    return config
##
## Gracefully sends SIGHUP to daemon
##
def refresh_config(daemon_name):
    config=get_daemon_config(daemon_name)
    # Read daemon PID from pidfile
    pidfile=config.get("main","pidfile",None)
    if not pidfile:
        return
    try:
        with open(pidfile) as f:
            pid=int(f.read().strip())
    except:
        return
    # Try to send SIGHUP to daemon
    try:
        os.kill(pid,signal.SIGHUP)
    except:
        pass



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## rack macro
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
from noc.kb.parsers.macros import Macro as MacroBase
import xml.parsers.expat
##
## RackSet representation
##
class RackSet(object):
    def __init__(self,id):
        self.id=id
        self.racks=[]
    ##
    ## Return a list of allocations for a rack
    ## allocation is a tuple of: top position, height, is empty space, title
    ##
    def compile_allocations(self,rack):
        allocations=sorted(rack.allocations,lambda x,y: -cmp(x.position,y.position))
        sp=[]
        if len(allocations)==0:
            sp+=[(rack.height,rack.height,True,None)]
        else:
            a=allocations.pop(0)
            empty_top=rack.height-a.position-a.height+1
            if empty_top:
                sp+=[(rack.height,empty_top,True,None)]
            sp+=[(a.position+a.height-1,a.height,False,a.id)]
            while allocations:
                last_a=a
                a=allocations.pop(0)
                empty_top=last_a.position-a.height-a.position
                if empty_top:
                    sp+=[(last_a.position-1,empty_top,True,None)]
                sp+=[(a.height+a.position-1,a.height,False,a.id)]
            if a.position>1:
                sp+=[(a.position-1,a.position-1,True,None)]
        return sp
    ##
    ## Render RackSet contents to HTML
    ##
    def render_html(self):
        if len(self.racks)==0: # Empty rackset
            return u""
        #return "<div>%s</div>"%"\n".join([r.render_html() for r in self.racks])
        self.height=max([r.height for r in self.racks]) # RackSet height is a maximal height of the racks
        # Fill RackSet matrix
        # RSM is a hash: row -> column -> {"colspan","rowspan","style","text"}
        rsm={}
        for i in range(1,self.height+1):
            rsm[i]={}
            for j in range(len(self.racks)):
                rsm[i][j*2]=None
                rsm[i][j*2+1]={"class":"ruler","text":str(i)}
        # Remove the tops of the racks if necessary
        for j in range(len(self.racks)):
            rh=self.racks[j].height
            if rh<self.height:
                for i in range(self.height,rh,-1):
                    rsm[i][j*2]=None
                    rsm[i][j*2+1]=None
                rsm[self.height][j*2]={"colspan":2,"rowspan":self.height-rh,"class":"emptytop"}
        # Place allocations
        for j in range(len(self.racks)):
            for a in self.compile_allocations(self.racks[j]):
                position,height,is_empty,title=a
                if is_empty:
                    style="empty"
                else:
                    style="occupied"
                if height==1:
                    rsm[position][j*2]={"class":style,"text":title}
                else:
                    rsm[position][j*2]={"class":style,"text":title,"rowspan":height}
                    for i in range(position-1,position-height,-1):
                        rsm[i][j*2]=None
        # Render the matrix
        out=["<table class='rackset'>"]
        if self.id:
            out+=["<caption>%s</caption>"%self.id]
        for i in range(self.height,0,-1):
            out+=["<tr>"]
            for j in range(len(self.racks)*2):
                v=rsm[i][j]
                if v:
                    td="<td "
                    for attr in ["colspan","rowspan","class"]:
                        if attr in v:
                            td+="%s='%s' "%(attr,v[attr])
                    td+=">"
                    if "text" in v and v["text"]:
                        td+=v["text"]
                    td+="</td>"
                    out+=[td]
            out+=["</tr>"]
        # Render rack titles
        out+=["<tr>"]
        for r in self.racks:
            if r.id:
                out+=["<td colspan='2' class='racklabel'>%s</td>"%r.id]
            else:
                out+=["<td colspan='2' class='racklabel'></td>"]
        out+=["</tr>"]
        out+=["</table>"]
        return u"\n".join(out)
##
## Rack Representation
##
class Rack(object):
    def __init__(self,rackset,id,height):
        self.rackset=rackset
        self.id=id
        self.height=height
        self.rackset.racks.append(self)
        self.allocations=[]
##
## Allocation representation
## Rendered to HTML by Rack.render_html
##
class Allocation(object):
    def __init__(self,rack,id,position,height):
        self.rack=rack
        self.id=id
        self.position=position
        self.height=height
        self.rack.allocations.append(self)
    
    def __repr__(self):
        return "Allocation: id=%s position=%s height=%s"%(self.id,self.position,self.height)
##
## Expat parser to render simple XML grammar
## Tag hirrarchy:
##     rackset attrs: id
##       `-> rack attrs: id, height
##              `-> allocation attrs: id, position, height
##
class XMLParser(object):
    def __init__(self,text):
        self.parser=xml.parsers.expat.ParserCreate()
        self.parser.StartElementHandler=self.start_element
        self.parser.EndElementHandler=self.end_element
        self.parser.CharacterDataHandler =self.char_data
        self.rackset=None
        self.last_rack=None
        if not text.startswith("<?"):
            text=u"<?xml version='1.0' encoding='utf-8' ?>\n"+text # Add missed XML prolog
        self.parser.Parse(unicode(text).encode("utf-8"))
    ##
    ## Called on tag opening
    ##
    def start_element(self,name,attrs):
        if name=="rackset":
            self.rackset=RackSet(id=attrs.get("id",None))
        elif name=="rack":
            height=attrs.get("height","1U").upper()
            if height.endswith("U"):
                height=height[:-1]
            self.last_rack=Rack(self.rackset,id=attrs.get("id",None),height=int(height))
        elif name=="allocation":
            height=attrs.get("height","1U").upper()
            if height.endswith("U"):
                height=height[:-1]
            allocation=Allocation(self.last_rack,id=attrs.get("id",None),height=int(height),position=int(attrs.get("position",0)))
    def end_element(self,name): pass
    def char_data(self,name): pass

##
## rack macro. XML contained in macro text
##
class Macro(MacroBase):
    name="rack"
    @classmethod
    def handle(cls,args,text):
        parser=XMLParser(text)
        return unicode(parser.rackset.render_html())



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Prefix-list-optimizing tree
##
## Tree is a binary tree containing prefixes.
## Each prefix is represented as sequence of the bits.
## All prefixes start with root.
## Zero bit on apropriative position in prefix means
## the prefix will be directed left from current node.
## The set bit means right direction.
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from noc.lib.ip import prefix_to_bin,bin_to_prefix

class Node(object):
    def __init__(self,parent=None,prefix=[]):
        self.parent=parent
        self.prefix=prefix
        self.children=[None,None]
        self.is_final=False
        
    # Binary prefix is a list of integers
    def append_binary_prefix(self,prefix):
        if not prefix:
            # Optimization #1
            # Remove existing specifics
            # When summary accepted
            if not self.is_final:
                self.release_children()
            # Mark this node as final
            self.is_final=True
            return
        # Optimization #2
        # Do not append specifics when summary exists
        if self.is_final:
            return
        d=prefix.pop(0)
        # Optimization #3
        # Do not append child when another final child exists.
        # Make this node final instead (summarize two prefixes)
        if not prefix:
            c=[c for c in self.children if c is not None]
            if len(c):
                if c[0].is_final:
                    self.release_children()
                    # Optimization #4
                    # When summarized prefixes try to summarize
                    # with siblings
                    n=self.parent
                    while n:
                        if len([c for c in n.children if c and c.is_final])==2:
                            n.release_children()
                            n=n.parent
                        else:
                            break
                    return
        # Cannot reduce prefix (yet). Append to the tree
        if not self.children[d]:
            self.children[d]=Node(self,self.prefix+[d])
        self.children[d].append_binary_prefix(prefix)
        
    def append_prefix(self,prefix):
        self.append_binary_prefix(prefix_to_bin(prefix))
        
    #
    # Extract remaining prefixes into list r
    #
    def extract_prefixes(self,r):
        if self.is_final:
            r.append(bin_to_prefix(self.prefix))
        [c.extract_prefixes(r) for c in self.children if c is not None]
        return r
        
    def release(self):
        self.parent=None
        self.release_children()
        
    def release_children(self):
        [c.release() for c in self.children if c is not None]
        self.children=[None,None]
        self.is_final=True

def optimize_prefix_list(prefix_list):
    """
    Optimize prefix list. Prefix list is a list of prefixes.
    Returns reduced list of prefixes
    
    Check optimization #1
    >>> optimize_prefix_list(["192.168.128.0/24","192.168.0.0/16"])
    ['192.168.0.0/16']
    Check optimization #2
    >>> optimize_prefix_list(["192.168.0.0/16","192.168.128.0/24"])
    ['192.168.0.0/16']
    Check optimization #3
    >>> optimize_prefix_list(["192.168.0.0/24","192.168.1.0/24"])
    ['192.168.0.0/23']
    Check optimization #4
    >>> optimize_prefix_list(["192.168.0.0/24","192.168.1.0/24","192.168.2.0/24","192.168.3.0/24"])
    ['192.168.0.0/22']
    >>> optimize_prefix_list(["192.168.%d.0/24"%i for i in range(16)])
    ['192.168.0.0/20']
     >>> optimize_prefix_list(["192.168.%d.0/24"%i for i in range(17)])
    ['192.168.0.0/20', '192.168.16.0/24']
    """
    n=Node()
    for p in prefix_list:
        n.append_prefix(p)
    r=[]
    n.extract_prefixes(r)
    return r



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from django.db import models
from django.db.models import Q
from django.core.mail import send_mail
from django.contrib.auth.models import User
from django.conf import settings
from noc.sa.profiles import profile_registry
from noc.settings import config
from noc.lib.url import URL
from noc.lib.fileutils import rewrite_when_differ,read_file,is_differ,in_dir
from noc.lib.validators import is_int
from noc.cm.vcs import vcs_registry
import os,datetime,stat,logging,sets,random
from noc.sa.models import Activator,AdministrativeDomain,ObjectGroup,ManagedObject
from noc.main.menu import Menu
from noc.sa.protocols.sae_pb2 import *
from noc.main.search import SearchResult

profile_registry.register_all()
vcs_registry.register_all()

#
OBJECT_TYPES=["config","dns","prefix-list","rpsl"]
OBJECT_TYPE_CHOICES=[(x,x) for x in OBJECT_TYPES]

class ObjectNotify(models.Model):
    class Meta:
        verbose_name="Object Notify"
        verbose_name_plural="Object Notifies"
    type=models.CharField("Type",max_length=16,choices=OBJECT_TYPE_CHOICES)
    administrative_domain=models.ForeignKey(AdministrativeDomain,verbose_name="Administrative Domain",blank=True,null=True)
    group=models.ForeignKey(ObjectGroup,verbose_name="Group",blank=True,null=True)
    emails=models.CharField("Emails",max_length=128)
    notify_immediately=models.BooleanField("Notify Immediately")
    notify_delayed=models.BooleanField("Notify Delayed")
    def __unicode__(self):
        return "(%s,%s,%s,%s)"%(self.type,self.administrative_domain,self.group,self.emails)

#
class Object(models.Model):
    class Meta:
        abstract=True
    repo_path=models.CharField("Repo Path",max_length=128,unique=True)
    #
    last_modified=models.DateTimeField("Last Modified",blank=True,null=True)
    #
    push_every=models.PositiveIntegerField("Push Every (secs)",default=86400,blank=True,null=True)
    next_push=models.DateTimeField("Next Push",blank=True,null=True)
    last_push=models.DateTimeField("Last Push",blank=True,null=True)
    #
    pull_every=models.PositiveIntegerField("Pull Every (secs)",default=86400,blank=True,null=True)
    next_pull=models.DateTimeField("Next Pull",blank=True,null=True)
    last_pull=models.DateTimeField("Last Pull",blank=True,null=True) # Updated by write() method
    
    def __unicode__(self):
        return "%s/%s"%(self.repo_name,self.repo_path)
    
    def _vcs(self):
        return vcs_registry.get(self.repo)
    vcs=property(_vcs)
        
    def save(self,*args,**kwargs):
        if self.repo_path and not in_dir(self.path,self.repo):
            raise Exception("Attempting to write outside of repo")
        mv=None
        if self._get_pk_val():
            old=self.__class__.objects.get(pk=self._get_pk_val())
            if old.repo_path!=self.repo_path:
                mv=(old.repo_path,self.repo_path)
        models.Model.save(self,*args,**kwargs)
        vcs=self.vcs
        if mv is not None and vcs.in_repo(mv[0]):
            vcs.mv(mv[0],mv[1])
        
    def _repo(self):
        return os.path.join(config.get("cm","repo"),self.repo_name)
    repo=property(_repo)

    def _path(self):
        return os.path.join(self.repo,self.repo_path)
    path=property(_path)
    
    #
    # If "data" differs from object's content in the repository
    # Write "data" to file and commit
    #
    def write(self,data):
        path=self.path
        if not in_dir(path,self.repo):
            raise Exception("Attempting to write outside of repo")
        is_new=not os.path.exists(path)
        now=datetime.datetime.now()
        if rewrite_when_differ(self.path,data):
            vcs=self.vcs
            if is_new:
                vcs.add(self.repo_path)
            vcs.commit(self.repo_path)
            self.last_modified=now
            self.on_object_changed()
        self.last_pull=now
        self.save()
    # Returns object's content
    # Or None if no content yet
    def _data(self):
        return read_file(self.path)
    data=property(_data)
    #
    def delete(self):
        if os.path.exists(self.repo_path):
            self.vcs.rm(self.path)
        super(Object,self).delete()

    def view_link(self):
        return "<A HREF='/cm/view/%s/%d/'>View</A>"%(self.repo_name,self.id)
    view_link.short_description="View"
    view_link.allow_tags=True

    def _revisions(self):
        return self.vcs.log(self.repo_path)
    revisions=property(_revisions)
    
    # Finds revision of the object and returns Revision
    def find_revision(self,rev):
        assert is_int(rev)
        for r in self.revisions:
            if r.revision==rev:
                return r
        raise Exception("Not found")
    
    def diff(self,rev1,rev2):
        return self.vcs.diff(self.repo_path,rev1,rev2)
        
    def get_revision(self,rev):
        return self.vcs.get_revision(self.repo_path,rev)
        
    @classmethod
    def get_object_class(self,repo):
        if repo=="config":
            return Config
        elif repo=="dns":
            return DNS
        elif repo=="prefix-list":
            return PrefixList
        elif repo=="rpsl":
            return RPSL
        else:
            raise Exception("Invalid repo '%s'"%repo)
    
    def change_notify_list(self,immediately=False,delayed=False):
        emails=sets.Set()
        for n in ObjectNotify.objects.filter(type=self.repo_name):
            if immediately and not n.notify_immediately:
                continue
            if delayed and not n.notify_delayed:
                continue
            emails.update(n.emails.split())
        return list(emails)
        
    def on_object_changed(self):
        emails=self.change_notify_list(immediately=True)
        if not emails:
            return
        revs=self.revisions
        now=datetime.datetime.now()
        if len(revs)==1:
            subject="NOC: Object '%s' was created"%str(self)
            message="The object %s was created at %s\n"%(str(self),now)
            message+="Object value follows:\n---------------------------\n%s\n-----------------------\n"%self.data
        else:
            subject="NOC: Object changed '%s'"%str(self)
            message="The object %s was changed at %s\n"%(str(self),now)
            message+="Object changes follows:\n---------------------------\n%s\n-----------------------\n"%self.diff(revs[1],revs[0])
        send_mail(subject=subject,message=message,from_email=settings.SERVER_EMAIL,recipient_list=emails,fail_silently=True)
    ##
    ##
    def push(self): pass
    def pull(self): pass
    #
    # Push all objects of the given type
    #
    @classmethod
    def global_push(self): pass
    #
    # Pull all objects of the given type
    #
    @classmethod
    def global_pull(self): pass
    #
    # Permission checks
    #
    def has_access(self,user):
        if user.is_superuser:
            return True
        return False
    ##
    ## Search engine
    ##
    @classmethod
    def search(cls,user,query,limit):
        for o in [o for o in cls.objects.all() if o.repo_path and o.has_access(user)]:
            data=o.data
            if query in o.repo_path: # If repo_path matches
                yield SearchResult(
                    url="/cm/view/%s/%d/"%(o.repo_name,o.id),
                    title="CM: "+unicode(o),
                    text=unicode(o),
                    relevancy=1.0, # No weighted search yes
                    )                
            elif data and query in data: # Dumb substring search in config
                idx=data.index(query)
                idx_s=max(0,idx-100)
                idx_e=min(len(data),idx+len(query)+100)
                text=data[idx_s:idx_e]
                yield SearchResult(
                    url="/cm/view/%s/%d/"%(o.repo_name,o.id),
                    title="CM: "+unicode(o),
                    text=text,
                    relevancy=1.0, # No weighted search yes
                    )
##
## Config
##
class Config(Object):
    class Meta:
        verbose_name="Config"
        verbose_name_plural="Configs"
    managed_object=models.OneToOneField(ManagedObject,verbose_name="Managed Object",unique=True)
    
    repo_name="config"
    def _profile(self):
        return profile_registry[self.profile_name]()
    profile=property(_profile)
    
    def pull(self,sae):
        def pull_callback(result=None,error=None):
            if error:
                if error.code==ERR_OVERLOAD:
                    timeout=config.getint("cm","timeout_overload")
                elif error.code==ERR_DOWN:
                    timeout=config.getint("cm","timeout_down")
                else:
                    timeout=config.getint("cm","timeout_error")
                variation=config.getint("cm","timeout_variation")
                timeout+=random.randint(-timeout/variation,timeout/variation) # Add jitter to avoid blocking by dead task
                self.next_pull=datetime.datetime.now()+datetime.timedelta(seconds=timeout)
                self.save()
                return            
            if self.pull_every:
                self.next_pull=datetime.datetime.now()+datetime.timedelta(seconds=self.pull_every)
                self.save()
            self.write(result)
        sae.script(self.managed_object,"%s.get_config"%self.managed_object.profile_name,pull_callback)
    def scripts_link(self):
        return "<A HREF='/sa/%d/scripts/'>Scripts</A>"%(self.id)
    scripts_link.short_description="Scripts"
    scripts_link.allow_tags=True
    ##
    ## Access control
    ##
    def has_access(self,user):
        return self.managed_object.has_access(user)
        
    @classmethod
    def queryset(cls,user):
        # Idiotic implementation
        ids=[o.id for o in cls.objects.all() if o.has_access(user)]
        return cls.objects.filter(id__in=ids)
    
    def change_notify_list(self,immediately=False,delayed=False):
        emails=sets.Set()
        for n in ObjectNotify.objects.filter(Q(type=self.repo_name)\
                    &(Q(administrative_domain__isnull=True)|Q(administrative_domain=self.managed_object.administrative_domain))\
                    &(Q(group__isnull=True)|Q(group__in=self.managed_object.groups.all))):
            if immediately and not n.notify_immediately:
                continue
            if delayed and not n.notify_delayed:
                continue
            emails.update(n.emails.split())
        return list(emails)

##
## PrefixList
##
class PrefixList(Object):
    class Meta:
        verbose_name="Prefix List"
        verbose_name_plural="Prefix Lists"
    repo_name="prefix-list"
    @classmethod
    def global_pull(cls):
        from noc.peer.builder import build_prefix_lists
        objects={}
        for o in PrefixList.objects.all():
            objects[o.repo_path]=o
        logging.debug("PrefixList.global_pull(): building prefix lists")
        for peering_point,pl_name,pl in build_prefix_lists():
            logging.debug("PrefixList.global_pull(): writing %s/%s (%d lines)"%(peering_point.hostname,pl_name,len(pl.split("\n"))))
            path=os.path.join(peering_point.hostname,pl_name)
            if path in objects:
                o=objects[path]
                del objects[path]
            else:
                o=PrefixList(repo_path=path)
                o.save()
            o.write(pl)
        for o in objects.values():
            o.delete()
##
## DNS
##
class DNS(Object):
    class Meta:
        verbose_name="DNS Object"
        verbose_name_plural="DNS Objects"
    repo_name="dns"
    @classmethod
    def global_pull(cls):
        from noc.dns.models import DNSZone,DNSServer
        
        objects={}
        changed={}
        for o in DNS.objects.exclude(repo_path__endswith="autozones.conf"):
            objects[o.repo_path]=o
        for z in DNSZone.objects.filter(is_auto_generated=True):
            for ns in z.profile.masters.all():
                path=os.path.join(ns.name,z.name)
                if path in objects:
                    o=objects[path]
                    del objects[path]
                else:
                    logging.debug("DNSHandler.global_pull: Creating object %s"%path)
                    o=DNS(repo_path=path)
                    o.save()
                if is_differ(o.path,z.zonedata(ns)):
                    changed[z]=None
        for o in objects.values():
            logging.debug("DNS.global_pull: Deleting object: %s"%o.repo_path)
            o.delete()
        for z in changed:
            logging.debug("DNS.global_pull: Zone %s changed"%z.name)
            z.serial=z.next_serial
            z.save()
            for ns in z.profile.masters.all():
                path=os.path.join(ns.name,z.name)
                o=DNS.objects.get(repo_path=path)
                o.write(z.zonedata(ns))
        for ns in DNSServer.objects.all():
            logging.debug("DNSHandler.global_pull: Includes for %s rebuilded"%ns.name)
            g=ns.generator_class()
            path=os.path.join(ns.name,"autozones.conf")
            try:
                o=DNS.objects.get(repo_path=path)
            except Object.DoesNotExist:
                o=DNS(repo_path=path)
                o.save()
            o.write(g.get_include(ns))
            
    @classmethod
    def global_push(self):
        from noc.dns.models import DNSZone
        nses={}
        for z in DNSZone.objects.filter(is_auto_generated=True):
            for ns in z.profile.masters.all():
                nses[ns.name]=ns
        for ns in nses.values():
            logging.debug("DNSHandler.global_push: provisioning %s"%ns.name)
            ns.provision_zones()
##
## RPSL
##
class RPSL(Object):
    class Meta:
        verbose_name="RPSL Object"
        verbose_name_plural="RPSL Objects"
    repo_name="rpsl"
    @classmethod
    def global_pull(cls):
        def global_pull_class(name,c,name_fun):
            objects={}
            for o in RPSL.objects.filter(repo_path__startswith=name+os.sep):
                objects[o.repo_path]=o
            for a in c.objects.all():
                if not a.rpsl:
                    continue
                path=os.path.join(name,name_fun(a))
                if path in objects:
                    o=objects[path]
                    del objects[path]
                else:
                    o=RPSL(repo_path=path)
                    o.save()
                o.write(a.rpsl)
            for o in objects.values():
                o.delete()
        from noc.peer.models import AS,ASSet,PeeringPoint
        from noc.dns.models import DNSZone
        logging.debug("RPSL.global_pull(): building RPSL")
        global_pull_class("inet-rtr",PeeringPoint,lambda a:a.hostname)
        global_pull_class("as",AS,lambda a:"AS%d"%a.asn)
        global_pull_class("as-set",ASSet,lambda a:a.name)
        global_pull_class("domain",DNSZone, lambda a:a.name)
    
    @classmethod
    def global_push(cls): pass
##
## Application Menu
##
class AppMenu(Menu):
    app="cm"
    title="Configuration Management"
    items=[
        ("Config",      "/admin/cm/config/",     "cm.change_config"),
        ("DNS Objects", "/admin/cm/dns/",        "cm.change_dns"),
        ("Prefix Lists","/admin/cm/prefixlist/", "cm.change_prefixlist"),
        ("RPSL Objects","/admin/cm/rpsl/",       "cm.change_rpsl"),
        ("Setup",[
            ("Object Notifies","/admin/cm/objectnotify/", "cm.change_objectnotify"),
        ])
    ]



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Power-over-Ethernet classification rules
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from noc.fm.rules.classification import ClassificationRule
from noc.fm.rules.classes.poe import *
##
## Cisco.IOS PoE Power Granted Syslog
##
class Cisco_IOS_PoE_Power_Granted_Syslog_Rule(ClassificationRule):
    name="Cisco.IOS PoE Power Granted Syslog"
    event_class=PoEPowerGranted
    preference=1000
    patterns=[
        (r"^profile$",r"^Cisco\.IOS$"),
        (r"^source$",r"^syslog$"),
        (r"^message$",r"%ILPOWER-5-POWER_GRANTED: Interface (?P<interface>.+): Power granted$"),
    ]
##
## Cisco.IOS PoE Power Disconnect Syslog
##
class Cisco_IOS_PoE_Power_Disconnect_Syslog_Rule(ClassificationRule):
    name="Cisco.IOS PoE Power Disconnect Syslog"
    event_class=PoEPowerDisconnect
    preference=1000
    patterns=[
        (r"^profile$",r"^Cisco\.IOS$"),
        (r"^source$",r"^syslog$"),
        (r"^message$",r": %ILPOWER-5-IEEE_DISCONNECT: Interface (?P<interface>.+): PD removed$"),
    ]



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from south.db import db
from noc.cm.models import *

class Migration:
    
    def forwards(self):
        
        # Model 'RPSL'
        db.create_table('cm_rpsl', (
            ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
            ('repo_path', models.CharField("Repo Path",max_length=128,unique=True)),
            ('push_every', models.PositiveIntegerField("Push Every (secs)",default=86400,blank=True,null=True)),
            ('next_push', models.DateTimeField("Next Push",blank=True,null=True)),
            ('last_push', models.DateTimeField("Last Push",blank=True,null=True)),
            ('pull_every', models.PositiveIntegerField("Pull Every (secs)",default=86400,blank=True,null=True)),
            ('next_pull', models.DateTimeField("Next Pull",blank=True,null=True)),
            ('last_pull', models.DateTimeField("Last Pull",blank=True,null=True))
        ))
        # Mock Models
        RPSL = db.mock_model(model_name='RPSL', db_table='cm_rpsl', db_tablespace='', pk_field_name='id', pk_field_type=models.AutoField)
        ObjectCategory = db.mock_model(model_name='ObjectCategory', db_table='cm_objectcategory', db_tablespace='', pk_field_name='id', pk_field_type=models.AutoField)
        
        # M2M field 'RPSL.categories'
        db.create_table('cm_rpsl_categories', (
            ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
            ('rpsl', models.ForeignKey(RPSL, null=False)),
            ('objectcategory', models.ForeignKey(ObjectCategory, null=False))
        )) 
        
        db.send_create_signal('cm', ['RPSL'])
    
    def backwards(self):
        db.delete_table('cm_rpsl')
        
        db.delete_table('cm_rpsl_categories')



#!/usr/bin/env python
##----------------------------------------------------------------------
## Distutils setup.py
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
from __future__ import with_statement
from distutils.core import setup
import distutils.command.sdist
from distutils.command.install import INSTALL_SCHEMES
import subprocess,os,sys

##
## Prefix to where noc to be installed
##
PREFIX="/opt/noc"

##
## Read actual noc version from VERSION file
##
def get_version():
    with open("VERSION") as f:
        return f.read().strip()
##
## Build distribution manifest
##
MANIFEST=[]
def get_manifest():
    global MANIFEST
    if not MANIFEST:
        if os.path.exists(".hg"):
            # Rebuild MANIFEST file every time mercurial repo found
            proc=subprocess.Popen(["hg","locate"],stdout=subprocess.PIPE,stderr=subprocess.PIPE)
            stdout,stderr=proc.communicate()
            with open("MANIFEST","w") as f:
                f.write(stdout+"MANIFEST\n")
        with open("MANIFEST") as f:
            MANIFEST=[n for n in f.read().splitlines() if not n.startswith(".hg")]
    return MANIFEST
##
## Return list of packages
##
def get_packages():
    return [""]+[f[:-12].replace(os.sep,".") for f in get_manifest() if f.endswith("__init__.py") and f!="__init__.py"]
##
## Return data files
##
def get_data():
    data={}
    for df in get_manifest():
        #if df.endswith(".py"):
        #    continue
        d,f=os.path.split(df)
        if d not in data:
            data[d]=[df]
        else:
            data[d].append(df)
    return data.items()
##
## Get file list for sdist from MANIFEST
##
class noc_sdist(distutils.command.sdist.sdist):
    def get_file_list(self):
        self.filelist.files=get_manifest()
##
## Monkeypatch distutils to install noc to the desired location
##
for scheme in INSTALL_SCHEMES.values():
    scheme['purelib']=PREFIX
    scheme['data']=PREFIX
##
## Pass control to the setuptools
##
setup(name="noc",
      version      = get_version(),
      description  = "Network Operation Center's OSS",
      author       = "Dmitry Volodin",
      author_email = "dvolodin7@google.com",
      url          = "http://nocproject.org/",
      license      = "BSD",
      long_description = """NOC Project is an Operation Support System (OSS) for telecom companies,
service providers, and enterprise Network Operation Centers (NOC).
Areas covered by NOC include fault management, service activation/provisioning, multi-VRF address space management,
configuration management, DNS provisioning, peering management, RPSL and BGP filter generation, and reporting.""",
      cmdclass     = {"sdist":noc_sdist},
      packages     = get_packages(),
      data_files   = get_data(),
      provides=["noc"],
      requires=[
        "psycopg2 (>= 2.0.5)",
        "Django (>= 1.0.2)",
        "South (>= 0.4)",
        "flup (>= 1.0)",
        "protobuf (>= 2.0.3)",
        "pysnmp (>= 4.1.7)",
        "pyasn1 (>= 0.0.6)",
        "pyke (>= 0.7)",
        "Sphinx (>= 0.5)",
#        "python-creole (>=0.1.0)",
      ],
     )



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from south.db import db
from noc.sa.models import *

class Migration:
    depends_on=(
        ("cm","0010_trap_source_ip"),
    )
    def forwards(self):
        
        # Model 'AdministrativeDomain'
        db.create_table('sa_administrativedomain', (
            ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
            ('name', models.CharField("Name",max_length=32,unique=True)),
            ('description', models.TextField("Description",null=True,blank=True))
        ))
        # Model 'ObjectGroup'
        db.create_table('sa_objectgroup', (
            ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
            ('name', models.CharField("Name",max_length=32,unique=True)),
            ('description', models.TextField("Description",null=True,blank=True))
        ))
        
        # Mock Models
        AdministrativeDomain = db.mock_model(model_name='AdministrativeDomain', db_table='sa_administrativedomain', db_tablespace='', pk_field_name='id', pk_field_type=models.AutoField)
        Activator = db.mock_model(model_name='Activator', db_table='sa_activator', db_tablespace='', pk_field_name='id', pk_field_type=models.AutoField)
        
        # Model 'ManagedObject'
        db.create_table('sa_managedobject', (
            ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
            ('name', models.CharField("Name",max_length=64,unique=True)),
            ('is_managed', models.BooleanField("Is Managed?",default=True)),
            ('administrative_domain', models.ForeignKey(AdministrativeDomain,verbose_name=AdministrativeDomain)),
            ('activator', models.ForeignKey(Activator,verbose_name=Activator)),
            ('profile_name', models.CharField("Profile",max_length=128,choices=profile_registry.choices)),
            ('scheme', models.IntegerField("Scheme",choices=[(TELNET,"telnet"),(SSH,"ssh"),(HTTP,"http")])),
            ('address', models.CharField("Address",max_length=64)),
            ('port', models.PositiveIntegerField("Port",blank=True,null=True)),
            ('user', models.CharField("User",max_length=32,blank=True,null=True)),
            ('password', models.CharField("Password",max_length=32,blank=True,null=True)),
            ('super_password', models.CharField("Super Password",max_length=32,blank=True,null=True)),
            ('remote_path', models.CharField("Path",max_length=32,blank=True,null=True)),
            ('trap_source_ip', models.IPAddressField("Trap Source IP",null=True)),
            ('trap_community', models.CharField("Trap Community",blank=True,null=True,max_length=64)),
            ('is_configuration_managed', models.BooleanField("Is Configuration Managed?",default=True)),
            ('repo_path', models.CharField("Repo Path",max_length=128,blank=True,null=True))
        ))
        # Mock Models
        ManagedObject = db.mock_model(model_name='ManagedObject', db_table='sa_managedobject', db_tablespace='', pk_field_name='id', pk_field_type=models.AutoField)
        ObjectGroup = db.mock_model(model_name='ObjectGroup', db_table='sa_objectgroup', db_tablespace='', pk_field_name='id', pk_field_type=models.AutoField)
        
        # M2M field 'ManagedObject.groups'
        db.create_table('sa_managedobject_groups', (
            ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
            ('managedobject', models.ForeignKey(ManagedObject, null=False)),
            ('objectgroup', models.ForeignKey(ObjectGroup, null=False))
        )) 
        
        # Mock Models
        User = db.mock_model(model_name='User', db_table='auth_user', db_tablespace='', pk_field_name='id', pk_field_type=models.AutoField)
        AdministrativeDomain = db.mock_model(model_name='AdministrativeDomain', db_table='sa_administrativedomain', db_tablespace='', pk_field_name='id', pk_field_type=models.AutoField)
        ObjectGroup = db.mock_model(model_name='ObjectGroup', db_table='sa_objectgroup', db_tablespace='', pk_field_name='id', pk_field_type=models.AutoField)
        
        # Model 'UserAccess'
        db.create_table('sa_useraccess', (
            ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
            ('user', models.ForeignKey(User,verbose_name=User)),
            ('administrative_domain', models.ForeignKey(AdministrativeDomain,verbose_name="Administrative Domain",blank=True,null=True)),
            ('group', models.ForeignKey(ObjectGroup,verbose_name="Group",blank=True,null=True)),
        ))
        
        db.send_create_signal('sa', ['AdministrativeDomain','ObjectGroup','ManagedObject','UserAccess'])
    
    def backwards(self):
        db.delete_table('sa_managedobject_groups')
        db.delete_table('sa_useraccess')
        db.delete_table('sa_managedobject')
        db.delete_table('sa_objectgroup')
        db.delete_table('sa_administrativedomain')



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from django.contrib import admin
from django import forms
from models import *
from noc.lib.fileutils import in_dir
from noc.settings import config
import os

class ActivatorAdmin(admin.ModelAdmin):
    list_display=["name","ip","is_active"]
    
class TaskScheduleAdmin(admin.ModelAdmin):
    list_display=["periodic_name","is_enabled","run_every","timeout","retries","retry_delay","next_run","retries_left"]

class AdministrativeDomainAdmin(admin.ModelAdmin):
    list_display=["name","description"]
    search_fields=["name","description"]

class ObjectGroupAdmin(admin.ModelAdmin):
    list_display=["name","description"]
    search_fields=["name","description"]

class ManagedObjectAdminForm(forms.ModelForm):
    class Meta:
        model=ManagedObject
    def clean_scheme(self):
        if "profile_name" not in self.cleaned_data:
            return self.cleaned_data["scheme"]
        profile=profile_registry[self.cleaned_data["profile_name"]]
        if self.cleaned_data["scheme"] not in profile.supported_schemes:
            raise forms.ValidationError("Selected scheme is not supported for profile '%s'"%self.cleaned_data["profile_name"])
        return self.cleaned_data["scheme"]
    # Check repo_path remains inside repo
    def clean_repo_path(self):
        repo=os.path.join(config.get("cm","repo"),"config")
        if not in_dir(os.path.join(repo,self.cleaned_data["repo_path"]),repo) or self.cleaned_data["repo_path"].startswith(os.sep):
            raise forms.ValidationError("Repo path must be relative path inside repo")
        return os.path.normpath(self.cleaned_data["repo_path"])
        
class ManagedObjectAdmin(admin.ModelAdmin):
    form=ManagedObjectAdminForm
    fieldsets=(
        (None,{
            "fields": ("name","is_managed","administrative_domain","activator","profile_name","description")
        }),
        ("Access",{
            "fields": ("scheme","address","port")
        }),
        ("Credentials",{
            "fields": ("user","password","super_password")
        }),
        ("SNMP",{
            "fields": ("snmp_ro","snmp_rw","trap_source_ip","trap_community")
        }),
        ("CM",{
            "fields": ("is_configuration_managed","repo_path")
        }),
        ("Groups", {
            "fields": ("groups",)
        }),
    )
    list_display=["name","is_managed","profile_name","address","administrative_domain","activator","is_configuration_managed","description","repo_path","scripts_link","config_link"]
    list_filter=["is_managed","is_configuration_managed","activator","administrative_domain","groups","profile_name"]
    search_fields=["name","address","repo_path","description"]
    object_class=ManagedObject
    ##
    ## Dirty hack to display PasswordInput in admin form
    ##
    def formfield_for_dbfield(self, db_field, **kwargs):
        if db_field.name in ["password","super_password"]:
            kwargs["widget"]=forms.widgets.PasswordInput
            return db_field.formfield(**kwargs)
        return super(ManagedObjectAdmin,self).formfield_for_dbfield(db_field,**kwargs)
    ##
    ## Row-level access control
    ##
    def has_change_permission(self,request,obj=None):
        if obj:
            return obj.has_access(request.user)
        else:
            return admin.ModelAdmin.has_delete_permission(self,request)
            
    def has_delete_permission(self,request,obj=None):
        return self.has_change_permission(request,obj)
        
    def queryset(self,request):
        return ManagedObject.queryset(request.user)
        
    def save_model(self, request, obj, form, change):
        if obj.can_change(request.user,form.cleaned_data["administrative_domain"],form.cleaned_data["groups"]):
            admin.ModelAdmin.save_model(self,request,obj,form,change)
        else:
            raise "Permission denied"

class UserAccessAdmin(admin.ModelAdmin):
    list_display=["user","administrative_domain","group"]
    list_filter=["user","administrative_domain","group"]

admin.site.register(Activator,            ActivatorAdmin)
admin.site.register(AdministrativeDomain, AdministrativeDomainAdmin)
admin.site.register(ObjectGroup,          ObjectGroupAdmin)
admin.site.register(ManagedObject,        ManagedObjectAdmin)
admin.site.register(UserAccess,           UserAccessAdmin)
admin.site.register(TaskSchedule,         TaskScheduleAdmin)




from south.db import db
from noc.kb.models import *

class Migration:
    
    def forwards(self):
        
        
        # Mock Models
        KBEntry = db.mock_model(model_name='KBEntry', db_table='kb_kbentry', db_tablespace='', pk_field_name='id', pk_field_type=models.AutoField)
        
        # Model 'KBGlobalBookmark'
        db.create_table('kb_kbglobalbookmark', (
            ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
            ('kb_entry', models.ForeignKey(KBEntry,verbose_name=KBEntry,unique=True))
        ))
        
        # Mock Models
        User = db.mock_model(model_name='User', db_table='auth_user', db_tablespace='', pk_field_name='id', pk_field_type=models.AutoField)
        KBEntry = db.mock_model(model_name='KBEntry', db_table='kb_kbentry', db_tablespace='', pk_field_name='id', pk_field_type=models.AutoField)
        
        # Model 'KBUserBookmark'
        db.create_table('kb_kbuserbookmark', (
            ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
            ('user', models.ForeignKey(User,verbose_name=User)),
            ('kb_entry', models.ForeignKey(KBEntry,verbose_name=KBEntry))
        ))
        db.create_index('kb_kbuserbookmark', ['user_id','kb_entry_id'], unique=True, db_tablespace='')
        
        
        db.send_create_signal('kb', ['KBGlobalBookmark','KBUserBookmark'])
    
    def backwards(self):
        db.delete_table('kb_kbuserbookmark')
        db.delete_table('kb_kbglobalbookmark')
        




from south.db import db
from noc.fm.models import *

class Migration:
    
    def forwards(self):
        
        
        # Mock Models
        EventClass = db.mock_model(model_name='EventClass', db_table='fm_eventclass', db_tablespace='', pk_field_name='id', pk_field_type=models.AutoField)
        EventPriority = db.mock_model(model_name='EventPriority', db_table='fm_eventpriority', db_tablespace='', pk_field_name='id', pk_field_type=models.AutoField)
        EventCategory = db.mock_model(model_name='EventCategory', db_table='fm_eventcategory', db_tablespace='', pk_field_name='id', pk_field_type=models.AutoField)
        
        # Model 'EventPostProcessingRule'
        db.create_table('fm_eventpostprocessingrule', (
            ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
            ('event_class', models.ForeignKey(EventClass,verbose_name="Event Class")),
            ('name', models.CharField("Name",max_length=64)),
            ('preference', models.IntegerField("Preference",default=1000)),
            ('is_active', models.BooleanField("Is Active",default=True)),
            ('description', models.TextField("Description",blank=True,null=True)),
            ('change_priority', models.ForeignKey(EventPriority,verbose_name="Change Priority to",blank=True,null=True)),
            ('change_category', models.ForeignKey(EventCategory,verbose_name="Change Category to",blank=True,null=True)),
            ('action', models.CharField("Action",max_length=1,choices=[("A","Make Active"),("C","Close"),("D","Drop")],default="A"))
        ))
        
        # Mock Models
        EventPostProcessingRule = db.mock_model(model_name='EventPostProcessingRule', db_table='fm_eventpostprocessingrule', db_tablespace='', pk_field_name='id', pk_field_type=models.AutoField)
        
        # Model 'EventPostProcessingRE'
        db.create_table('fm_eventpostprocessingre', (
            ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
            ('rule', models.ForeignKey(EventPostProcessingRule,verbose_name="Event Post-Processing Rule")),
            ('var_re', models.CharField("Var RE",max_length=256)),
            ('value_re', models.CharField("Value RE",max_length=256))
        ))
        
        db.send_create_signal('fm', ['EventPostProcessingRule','EventPostProcessingRE'])
    
    def backwards(self):
        db.delete_table('fm_eventpostprocessingre')
        db.delete_table('fm_eventpostprocessingrule')
        



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Cisco IOS specific memory-related rules
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from noc.fm.rules.classification import ClassificationRule
from noc.fm.rules.classes.memory import *
##
## Cisco.IOS Memory Allocation Failed SYSLOG
##
class Cisco_IOS_Memory_Allocation_Failed_SYSLOG_Rule(ClassificationRule):
    name="Cisco.IOS Memory Allocation Failed SYSLOG"
    event_class=MemoryAllocationError
    preference=1000
    patterns=[
        (r"^source$",r"^syslog$"),
        (r"^profile$",r"^Cisco\.IOS$"),
        (r"^message$",r"%SYS-2-(?:MALLOCFAIL|CHUNKEXPANDFAIL)"),
    ]
##
## Cisco.IOS Memory Allocation Failed SYSLOG SNMP
##
class Cisco_IOS_Memory_Allocation_Failed_SYSLOG_SNMP_Rule(ClassificationRule):
    name="Cisco.IOS Memory Allocation Failed SYSLOG SNMP"
    event_class=MemoryAllocationError
    preference=1000
    required_mibs=["CISCO-SYSLOG-MIB"]
    patterns=[
        (r"^source$",r"^SNMP Trap$"),
        (r"^profile$",r"^Cisco\.IOS$"),
        (r"^1\.3\.6\.1\.6\.3\.1\.1\.4\.1\.0$",r"^1\.3\.6\.1\.4\.1\.9\.9\.41\.2\.0\.1$"),
        (r"^1\.3\.6\.1\.4\.1\.9\.9\.41\.1\.2\.3\.1\.2\.\d+$",r"^SYS$"),
        (r"^1\.3\.6\.1\.4\.1\.9\.9\.41\.1\.2\.3\.1\.4\.\d+$",r"^(?:CHUNKEXPANDFAIL|MALLOCFAIL)$"),
    ]



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Authentication classification rules
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from noc.fm.rules.classification import ClassificationRule,Expression
from noc.fm.rules.classes.auth import *
##
## Authentication Failure SNMP
## Provides IP address
##
class Authentication_Failure_SNMP_Rule(ClassificationRule):
    name="Authentication Failure SNMP"
    event_class=AuthenticationFailure
    preference=1000
    required_mibs=["SNMPv2-MIB"]
    patterns=[
        (r"^source$",r"^SNMP Trap$"),
        (r"^1\.3\.6\.1\.6\.3\.1\.1\.4\.1\.0$",r"^1\.3\.6\.1\.6\.3\.1\.1\.5\.5$"),
        (r"^1\.3\.6\.1\.4\.1\.9\.2\.1\.5\.0$",r"(?P<bin_ip>....)"),
        Expression("ip","decode_ipv4(bin_ip)"),
    ]
##
## Authentication Failure Simple SNMP
## Matches when no IP address provided
##
class Authentication_Failure_Fallback_SNMP_Rule(ClassificationRule):
    name="Authentication Failure Fallback SNMP"
    event_class=AuthenticationFailure
    preference=2000
    required_mibs=["SNMPv2-MIB"]
    patterns=[
        (r"^source$",r"^SNMP Trap$"),
        (r"^1\.3\.6\.1\.6\.3\.1\.1\.4\.1\.0$",r"^1\.3\.6\.1\.6\.3\.1\.1\.5\.5$"), # authenticationFailure
    ]



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## HTTP Probe
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from noc.lib.nbsocket import ConnectedTCPSocket
import noc.pm.probes
import urlparse,time
##
##
##
class HTTPProbeSocket(ConnectedTCPSocket):
    TTL=5
    def __init__(self,probe,address,port):
        self.probe=probe
        self.address=address
        self.port=port
        self.connect_time=None
        self.response_time=None
        url=self.probe.get("url","%s:%d"%(address,port))
        self.url=urlparse.urlparse(url)
        self.start_time=time.time()
        super(HTTPProbeSocket,self).__init__(probe.factory,address,port)
        
    def on_connect(self):
        self.connect_time=time.time()
        self.write("GET %(path)s HTTP/1.1\r\nHost: %(host)s\r\nConnection: close\r\nUser-Agent: %(uagent)s\r\nReferer: none\r\n\r\n"%{
            "path"   : self.url.path,
            "host"   : self.url.netloc,
            "uagent" : self.probe.get("user_agent","NOC SLA Monitor")
        })
        
    def on_close(self):
        self.response_time=time.time()
        if self.connect_time:
            self.probe.feed_result(self.address,{
                "status":True,
                "connect_time" : self.connect_time-self.start_time,
                "response_time": self.response_time-self.connect_time,
                "total_time"   : self.response_time-self.start_time})
        else:
            self.probe.feed_result(self.address,{"status":False})

    def on_read(self,data):
        pass
##
##
##
class HTTPProbe(noc.pm.probes.Probe):
    name="http"
    def run(self):
        for ip in [x.strip() for x in self.get("objects","").split(",")]:
            s=HTTPProbeSocket(self,ip,80)



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Administrative interface for Main application
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from django.contrib import admin
from noc.main.models import *

##
## Admin for Language
##
class LanguageAdmin(admin.ModelAdmin):
    list_display=["name","native_name","is_active"]
    search_fields=["name","native_name"]
    list_filter=["is_active"]
##
## Admin for MIME Types
##
class MIMETypeAdmin(admin.ModelAdmin):
    list_display=["extension","mime_type"]
    search_fields=["extension","mime_type"]
##
## Register administrative interfaces
##
admin.site.register(Language, LanguageAdmin)
admin.site.register(MIMEType, MIMETypeAdmin)



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Port events
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from noc.fm.rules.classes import EventClass,Var
##
## Port Trunk Status Change
##
class PortTrunkStatusChange(EventClass):
    name     = "Port Trunk Status Change"
    category = "NETWORK"
    priority = "WARNING"
    subject_template="Port {{ifindex}} changed trunk status to {{status}}"
    body_template="""Port {{ifindex}} changed trunk status to {{status}}"""
    repeat_suppression=False
    repeat_suppression_interval=3600
    trigger=None
    class Vars:
        ifindex=Var(required=True,repeat=False)
        status=Var(required=True,repeat=False)




from south.db import db
from noc.fm.models import *

class Migration:
    
    def forwards(self):
        
        
        # Mock Models
        MIB = db.mock_model(model_name='MIB', db_table='fm_mib', db_tablespace='', pk_field_name='id', pk_field_type=models.AutoField)
        MIB = db.mock_model(model_name='MIB', db_table='fm_mib', db_tablespace='', pk_field_name='id', pk_field_type=models.AutoField)
        
        # Model 'MIBDependency'
        db.create_table('fm_mibdependency', (
            ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
            ('mib', models.ForeignKey(MIB,verbose_name=MIB)),
            ('requires_mib', models.ForeignKey(MIB,verbose_name="Requires MIB",related_name="requiredbymib_set"))
        ))
        db.create_index('fm_mibdependency', ['mib_id','requires_mib_id'], unique=True, db_tablespace='')
        
        
        db.send_create_signal('fm', ['MIBDependency'])
    
    def backwards(self):
        db.delete_table('fm_mibdependency')
        



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Populate Languiages table with ISO639-1 codes
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------

from south.db import db
from noc.main.models import *

LANGUAGES=[
    ("Afar",            "Afaraf"),
    ("Abkhazian",       "Аҧсуа"),
    ("Avestan",         "avesta"),
    ("Afrikaans",       "Afrikaans"),
    ("Akan",            "Akan"),
    ("Amharic",         "አማርኛ"),
    ("Aragonese",       "Aragonés"),
    ("Arabic",          "العربية"),
    ("Assamese",         "অসমীয়া"),
    ("Avaric",          "авар мацӀ"),
    ("Aymara",          "aymar aru"),
    ("Azerbaijani",     "azərbaycan dili"),
    ("Bashkir",         "башҡорт теле"),
    ("Belarusian",      "Беларуская"),
    ("Bulgarian",       "български език"),
    ("Bihari",          "भोजपुरी"),
    ("Bislama",         "Bislama"),
    ("Bambara",         "bamanankan"),
    ("Bengali",         "বাংলা"),
    ("Tibetan",         "བོད་ཡིག"),
    ("Breton",          "brezhoneg"),
    ("Bosnian",         "bosanski jezik"),
    ("Catalan",         "Català"),
    ("Chechen",         "нохчийн мотт"),
    ("Chamorro",        "Chamoru"),
    ("Corsican",        "corsu; lingua corsa"),
    ("Cree",            "ᓀᐦᐃᔭᐍᐏᐣ"),
    ("Czech",           "česky"),
    ("Church Slavic",   "ѩзыкъ словѣньскъ"),
    ("Chuvash",         "чӑваш чӗлхи"),
    ("Welsh",           "Cymraeg"),
    ("Danish",          "Dansk"),
    ("German",          "Deutsch"),
    ("Divehi",           "ދިވެހި"),
    ("Dzongkha",        "རྫོང་ཁ"),
    ("Ewe",             "Ɛʋɛgbɛ"),
    ("Greek",           "Ελληνικά"),
    ("English",         "English"),
    ("Esperanto",       "Esperanto"),
    ("Spanish",         "Español;"),
    ("Estonian",        "Eesti"),
    ("Basque",          "euskara"),
    ("Persian",         "فارسی"),
    ("Fulah",           "Fulfulde"),
    ("Finnish",         "Suomi"),
    ("Fijian",          "Vosa Vakaviti"),
    ("Faroese",         "Føroyskt"),
    ("French",          "Français"),
    ("Western Frisian", "Frysk"),
    ("Irish",           "Gaeilge"),
    ("Scottish Gaelic", "Gàidhlig"),
    ("Galician",        "Galego"),
    ("Guaraní",         "Avañe'ẽ"),
    ("Gujarati",        "ગુજરાતી"),
    ("Manx",            "Gaelg"),
    ("Hausa",           "هَوُسَ"),
    ("Hebrew",          "עברית"),
    ("Hindi",           "हिन्दी"),
    ("Hiri Motu",       "Hiri Motu"),
    ("Croatian",        "Hrvatski"),
    ("Haitian",         "Kreyòl ayisyen"),
    ("Hungarian",       "Magyar"),
    ("Armenian",        "Հայերեն"),
    ("Herero",          "Otjiherero"),
    ("Interlingua",     "Interlingua"),
    ("Indonesian",      "Bahasa Indonesia"),
    ("Interlingue",     "Interlingue"),
    ("Igbo",            "Igbo"),
    ("Sichuan Yi",      "ꆇꉙ"),
    ("Inupiaq",         "Iñupiaq"),
    ("Ido",             "Ido"),
    ("Icelandic",       "Íslenska"),
    ("Italian",         "Italiano"),
    ("Inuktitut",       "ᐃᓄᒃᑎᑐᑦ"),
    ("Japanese",        "日本語"),
    ("Javanese",        "basa Jawa"),
    ("Georgian",        "ქართული"),
    ("Kongo",           "KiKongo"),
    ("Kikuyu",          "Gĩkũyũ"),
    ("Kwanyama",        "Kuanyama"),
    ("Kazakh",          "Қазақ тілі"),
    ("Kalaallisut",     "kalaallisut; kalaallit oqaasii"),
    ("Khmer",           "ភាសាខ្មែរ"),
    ("Kannada",         "ಕನ್ನಡ"),
    ("Korean",          "한국어"),
    ("Kanuri",          "Kanuri"),
    ("Kashmiri",        "कश्मीरी"),
    ("Kurdish",         "كوردی‎"),
    ("Komi",            "коми кыв"),
    ("Cornish",         "Kernewek"),
    ("Kirghiz",         "кыргыз тили"),
    ("Latin",           "Latine"),
    ("Luxembourgish",   "Lëtzebuergesch"),
    ("Ganda",           "Luganda"),
    ("Limburgish",      "Limburgs"),
    ("Lingala",         "Lingála"),
    ("Lao",             "ພາສາລາວ"),
    ("Lithuanian",      "Lietuvių kalba"),
    ("Latvian",         "Latviešu valoda"),
    ("Malagasy",        "Malagasy fiteny"),
    ("Marshallese",     "Kajin M̧ajeļ"),
    ("Māori",           "te reo Māori"),
    ("Macedonian",      "македонски јазик"),
    ("Malayalam",        "മലയാളം"),
    ("Mongolian",       "Монгол"),
    ("Marathi",         "मराठी"),
    ("Malay",           "بهاس ملايو‎"),
    ("Maltese",         "Malti"),
    ("Burmese",          "ဗမာစာ"),
    ("Nauru",           "Ekakairũ Naoero"),
    ("Norwegian Bokmål","Norsk bokmål"),
    ("North Ndebele",   "isiNdebele"),
    ("Nepali",          "नेपाली"),
    ("Ndonga",          "Owambo"),
    ("Dutch",           "Nederlands"),
    ("Norwegian Nynorsk","Norsk nynorsk"),
    ("Norwegian",       "Norsk"),
    ("South Ndebele",   "isiNdebele"),
    ("Navajo",          "Dinékʼehǰí"),
    ("Chichewa",        "chiCheŵa"),
    ("Occitan",         "Occitan"),
    ("Ojibwa",          "ᐊᓂᔑᓈᐯᒧᐎᓐ"),
    ("Oromo",           "Afaan Oromoo"),
    ("Oriya",           "ଓଡ଼ିଆ"),
    ("Ossetian",        "Ирон æвзаг"),
    ("Panjabi",         "ਪੰਜਾਬੀ"),
    ("Pāli",            "पाऴि"),
    ("Polish",          "Polski"),
    ("Pashto",          "پښتو"),
    ("Portuguese",      "Português"),
    ("Quechua",         "Runa Simi"),
    ("Raeto-Romance",   "rumantsch grischun"),
    ("Kirundi",         "kiRundi"),
    ("Romanian",        "română"),
    ("Russian",         "Русский"),
    ("Kinyarwanda",     "Ikinyarwanda"),
    ("Sanskrit",        "संस्कृतम्"),
    ("Sardinian",       "Sardu"),
    ("Sindhi",          "सिन्धी"),
    ("Northern Sami",   "Davvisámegiella"),
    ("Sango",           "yângâ tî sängö"),
    ("Serbo-Croatian",  "Српскохрватски"),
    ("Sinhala",          "සිංහල"),
    ("Slovak",          "Slovenčina"),
    ("Slovenian",       "Slovenščina"),
    ("Samoan",          "gagana fa'a Samoa"),
    ("Shona",           "chiShona"),
    ("Somali",          "Soomaaliga"),
    ("Albanian",        "Shqip"),
    ("Serbian",         "српски језик"),
    ("Swati",           "SiSwati"),
    ("Southern Sotho",  "Sesotho"),
    ("Sundanese",       "Basa Sunda"),
    ("Swedish",         "Svenska"),
    ("Swahili",         "Kiswahili"),
    ("Tamil",           "தமிழ்"),
    ("Telugu",           "తెలుగు"),
    ("Tajik",           "тоҷикӣ"),
    ("Thai",            "ไทย"),
    ("Tigrinya",         "ትግርኛ"),
    ("Turkmen",         "Түркмен"),
    ("Tagalog",         "Tagalog"),
    ("Tswana",          "Setswana"),
    ("Tonga",           "faka Tonga"),
    ("Turkish",         "Türkçe"),
    ("Tsonga",          "Xitsonga"),
    ("Tatar",           "татарча"),
    ("Twi",             "Twi"),
    ("Tahitian",        "Reo Mā`ohi"),
    ("Uighur",          "Uyƣurqə"),
    ("Ukrainian",       "Українська"),
    ("Urdu",            "اردو"),
    ("Uzbek",           "Ўзбек"),
    ("Venda",           "Tshivenḓa"),
    ("Vietnamese",      "Tiếng Việt"),
    ("Volapük",         "Volapük"),
    ("Walloon",         "Walon"),
    ("Wolof",           "Wollof"),
    ("Xhosa",           "isiXhosa"),
    ("Yiddish",         "ייִדיש"),
    ("Yoruba",          "Yorùbá"),
    ("Zhuang",          "Saɯ cueŋƅ"),
    ("Chinese",         "中文"),
    ("Zulu",            "isiZulu"),
]

class Migration:
    
    def forwards(self):
        for lang,native in LANGUAGES:
            db.execute("INSERT INTO main_language(name,native_name,is_active) VALUES(%s,%s,%s)",[lang,native,lang=="English"])
    
    def backwards(self):
        "Write your backwards migration here"



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
import noc.sa.periodic

class Task(noc.sa.periodic.Task):
    name="cm.prefix_list_pull"
    description=""
    
    def execute(self):
        from noc.cm.models import PrefixList
        PrefixList.global_pull()
        return True
